=== added file 'bin/ivle-cloneworksheets'
--- bin/ivle-cloneworksheets	1970-01-01 00:00:00 +0000
+++ bin/ivle-cloneworksheets	2009-02-25 02:01:45 +0000
@@ -0,0 +1,70 @@
+#!/usr/bin/python
+# IVLE - Informatics Virtual Learning Environment
+# Copyright (C) 2007-2009 The University of Melbourne
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+
+# Author: Will Grant
+
+import os
+import sys
+
+from ivle.database import get_store, Subject, Semester, Offering, Worksheet
+import ivle.worksheet
+
+def die(error):
+    print >> sys.stderr, '%s: %s' % (os.path.basename(sys.argv[0]), error)
+    sys.exit(1)
+
+if len(sys.argv) != 7:
+    print >> sys.stderr, "usage: %s <oldsubjectcode> <oldyear> <oldsemester> "\
+                         "<newsubjectcode> <newyear> <newsemester>" \
+                         % os.path.basename(sys.argv[0])
+    sys.exit()
+
+store = get_store()
+
+src = store.find(Offering,
+                 Subject.code == unicode(sys.argv[1]),
+                 Semester.year == unicode(sys.argv[2]),
+                 Semester.semester == unicode(sys.argv[3]),
+                 Offering.subject_id == Subject.id,
+                 Offering.semester_id == Semester.id).one()
+
+dst = store.find(Offering,
+                 Subject.code == unicode(sys.argv[4]),
+                 Semester.year == unicode(sys.argv[5]),
+                 Semester.semester == unicode(sys.argv[6]),
+                 Offering.subject_id == Subject.id,
+                 Offering.semester_id == Semester.id).one()
+
+if dst.worksheets.count() > 0:
+    die('%r already has worksheets - doing nothing' % dst)
+
+for oldws in src.worksheets:
+    newws = Worksheet()
+    newws.seq_no = oldws.seq_no
+    newws.identifier = oldws.identifier
+    newws.name = oldws.name
+    newws.assessable = oldws.assessable
+    newws.data = oldws.data
+    newws.format = oldws.format
+    newws.offering = dst
+    store.add(newws)
+    ivle.worksheet.update_exerciselist(newws)
+
+print >> sys.stderr, 'copied %d worksheets from %r to %r' \
+                     % (dst.worksheets.count(), src, dst)
+store.commit()

=== modified file 'bin/ivle-enrol'
--- bin/ivle-enrol	2009-02-02 04:00:25 +0000
+++ bin/ivle-enrol	2009-02-25 02:59:48 +0000
@@ -25,15 +25,17 @@
 import os
 import sys
 
-from ivle.database import get_store, Subject, Semester, Offering, User, \
-                          AlreadyEnrolledError
+from ivle.database import get_store, Subject, Semester, Offering, User
 
 def die(error):
     print >> sys.stderr, '%s: %s' % (os.path.basename(sys.argv[0]), error)
     sys.exit(1)
 
-if len(sys.argv) != 5:
-    print >> sys.stderr, "usage: %s <login> <subjectcode> <year> <semester>" \
+if len(sys.argv) not in (5, 6):
+    print >> sys.stderr, '''usage: %s <login> <subjectcode> <year> <semester> [role]
+
+'role' should be one of student (the default), tutor or lecturer.
+If an enrolment already exists, it will be reactivated and its role set.''' \
                          % os.path.basename(sys.argv[0])
     sys.exit()
 
@@ -52,11 +54,10 @@
 if not offering:
     die("subject offering not found")
 
-if not offering.semester.active:
-    die("cannot enrol - semester inactive")
-    
+role = unicode(sys.argv[5]) if len(sys.argv) == 6 else u'student'
+
 try:
-    offering.enrol(user)
+    offering.enrol(user, role)
 except AlreadyEnrolledError:
     die("already enrolled")
 

=== modified file 'bin/ivle-makeuser'
--- bin/ivle-makeuser	2009-02-16 23:53:08 +0000
+++ bin/ivle-makeuser	2009-02-24 13:29:44 +0000
@@ -41,7 +41,7 @@
 # Requireds and optionals will be used to display the usage message
 # AND do argument processing
 # The names here must correspond to the fields in the database.
-requireds = ["login", "fullname", "rolenm"]
+requireds = ["login", "fullname"]
 optionals = [
     ('p', 'password', "Cleartext password for this user"),
     ('n', 'nick', "Display name (defaults to <fullname>)"),

=== added file 'image-source/h1-bg.svg'
--- image-source/h1-bg.svg	1970-01-01 00:00:00 +0000
+++ image-source/h1-bg.svg	2009-02-24 07:43:05 +0000
@@ -0,0 +1,133 @@
+<?xml version="1.0" encoding="UTF-8" standalone="no"?>
+<!-- Created with Inkscape (http://www.inkscape.org/) -->
+<svg
+   xmlns:dc="http://purl.org/dc/elements/1.1/"
+   xmlns:cc="http://creativecommons.org/ns#"
+   xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#"
+   xmlns:svg="http://www.w3.org/2000/svg"
+   xmlns="http://www.w3.org/2000/svg"
+   xmlns:xlink="http://www.w3.org/1999/xlink"
+   xmlns:sodipodi="http://sodipodi.sourceforge.net/DTD/sodipodi-0.dtd"
+   xmlns:inkscape="http://www.inkscape.org/namespaces/inkscape"
+   width="500"
+   height="16"
+   id="svg2"
+   sodipodi:version="0.32"
+   inkscape:version="0.46"
+   version="1.0"
+   sodipodi:docname="file-pathbar-bg.svg"
+   inkscape:output_extension="org.inkscape.output.svg.inkscape"
+   inkscape:export-filename="/home/matt/uni/ivle/new-ui/ivle/webapp/filesystem/browser/media/chrome/pathbar-bg.png"
+   inkscape:export-xdpi="90"
+   inkscape:export-ydpi="90">
+  <defs
+     id="defs4">
+    <linearGradient
+       id="linearGradient3211">
+      <stop
+         style="stop-color:#ffcc66;stop-opacity:1;"
+         offset="0"
+         id="stop3213" />
+      <stop
+         style="stop-color:#ffcc66;stop-opacity:0;"
+         offset="1"
+         id="stop3215" />
+    </linearGradient>
+    <linearGradient
+       id="linearGradient3161">
+      <stop
+         style="stop-color:#ebdb2e;stop-opacity:1;"
+         offset="0"
+         id="stop3163" />
+      <stop
+         style="stop-color:#d0e916;stop-opacity:0;"
+         offset="1"
+         id="stop3165" />
+    </linearGradient>
+    <inkscape:perspective
+       sodipodi:type="inkscape:persp3d"
+       inkscape:vp_x="0 : 526.18109 : 1"
+       inkscape:vp_y="0 : 1000 : 0"
+       inkscape:vp_z="744.09448 : 526.18109 : 1"
+       inkscape:persp3d-origin="372.04724 : 350.78739 : 1"
+       id="perspective10" />
+    <inkscape:perspective
+       sodipodi:type="inkscape:persp3d"
+       inkscape:vp_x="0 : 526.18109 : 1"
+       inkscape:vp_y="0 : 1000 : 0"
+       inkscape:vp_z="744.09448 : 526.18109 : 1"
+       inkscape:persp3d-origin="372.04724 : 350.78739 : 1"
+       id="perspective5668" />
+    <inkscape:perspective
+       id="perspective5042"
+       inkscape:persp3d-origin="372.04724 : 350.78739 : 1"
+       inkscape:vp_z="744.09448 : 526.18109 : 1"
+       inkscape:vp_y="0 : 1000 : 0"
+       inkscape:vp_x="0 : 526.18109 : 1"
+       sodipodi:type="inkscape:persp3d" />
+    <linearGradient
+       id="linearGradient5587">
+      <stop
+         id="stop5589"
+         offset="0"
+         style="stop-color:#ffdd88;stop-opacity:1;" />
+      <stop
+         id="stop5591"
+         offset="1"
+         style="stop-color:#ffffff;stop-opacity:0;" />
+    </linearGradient>
+    <linearGradient
+       inkscape:collect="always"
+       xlink:href="#linearGradient3211"
+       id="linearGradient3217"
+       x1="31.642857"
+       y1="342.86218"
+       x2="522.5155"
+       y2="342.86218"
+       gradientUnits="userSpaceOnUse" />
+  </defs>
+  <sodipodi:namedview
+     id="base"
+     pagecolor="#ffffff"
+     bordercolor="#666666"
+     borderopacity="1.0"
+     gridtolerance="10000"
+     guidetolerance="10"
+     objecttolerance="10"
+     inkscape:pageopacity="0.0"
+     inkscape:pageshadow="2"
+     inkscape:zoom="1"
+     inkscape:cx="265.93878"
+     inkscape:cy="-32.614375"
+     inkscape:document-units="px"
+     inkscape:current-layer="layer1"
+     showgrid="false"
+     inkscape:window-width="1280"
+     inkscape:window-height="725"
+     inkscape:window-x="0"
+     inkscape:window-y="25" />
+  <metadata
+     id="metadata7">
+    <rdf:RDF>
+      <cc:Work
+         rdf:about="">
+        <dc:format>image/svg+xml</dc:format>
+        <dc:type
+           rdf:resource="http://purl.org/dc/dcmitype/StillImage" />
+      </cc:Work>
+    </rdf:RDF>
+  </metadata>
+  <g
+     inkscape:label="Layer 1"
+     inkscape:groupmode="layer"
+     id="layer1"
+     transform="translate(-31.881123,-332.81473)">
+    <rect
+       style="opacity:1;fill:url(#linearGradient3217);fill-opacity:1;fill-rule:nonzero;stroke:none;stroke-width:0.47653185999999997;stroke-linecap:butt;stroke-linejoin:miter;marker:none;marker-start:none;marker-mid:none;marker-end:none;stroke-miterlimit:4;stroke-dasharray:none;stroke-dashoffset:0;stroke-opacity:1;visibility:visible;display:inline;overflow:visible;enable-background:accumulate"
+       id="rect2383"
+       width="499.6011"
+       height="16"
+       x="31.881123"
+       y="332.81473" />
+  </g>
+</svg>

=== removed file 'image-source/title-old-1.svg'
--- image-source/title-old-1.svg	2009-02-24 02:02:03 +0000
+++ image-source/title-old-1.svg	1970-01-01 00:00:00 +0000
@@ -1,235 +0,0 @@
-<?xml version="1.0" encoding="UTF-8" standalone="no"?>
-<!-- Created with Inkscape (http://www.inkscape.org/) -->
-<svg
-   xmlns:dc="http://purl.org/dc/elements/1.1/"
-   xmlns:cc="http://creativecommons.org/ns#"
-   xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#"
-   xmlns:svg="http://www.w3.org/2000/svg"
-   xmlns="http://www.w3.org/2000/svg"
-   xmlns:sodipodi="http://sodipodi.sourceforge.net/DTD/sodipodi-0.dtd"
-   xmlns:inkscape="http://www.inkscape.org/namespaces/inkscape"
-   width="470"
-   height="50"
-   id="svg3397"
-   sodipodi:version="0.32"
-   inkscape:version="0.46"
-   sodipodi:docname="title-1.svg"
-   inkscape:output_extension="org.inkscape.output.svg.inkscape"
-   version="1.0"
-   inkscape:export-filename="/home/matt/uni/ivle/setup-stuff/ivle/webapp/coremedia/images/chrome/title.png"
-   inkscape:export-xdpi="90"
-   inkscape:export-ydpi="90"
-   style="display:inline">
-  <defs
-     id="defs3399">
-    <inkscape:perspective
-       sodipodi:type="inkscape:persp3d"
-       inkscape:vp_x="0 : 526.18109 : 1"
-       inkscape:vp_y="0 : 1000 : 0"
-       inkscape:vp_z="744.09448 : 526.18109 : 1"
-       inkscape:persp3d-origin="372.04724 : 350.78739 : 1"
-       id="perspective3405" />
-    <filter
-       inkscape:collect="always"
-       id="filter4841"
-       x="-0.40412903"
-       width="1.8082581"
-       y="-0.3915"
-       height="1.783">
-      <feGaussianBlur
-         inkscape:collect="always"
-         stdDeviation="2.6487424"
-         id="feGaussianBlur4843" />
-    </filter>
-    <filter
-       inkscape:collect="always"
-       id="filter4845"
-       x="-0.40412903"
-       width="1.8082581"
-       y="-0.3915"
-       height="1.783">
-      <feGaussianBlur
-         inkscape:collect="always"
-         stdDeviation="2.6487424"
-         id="feGaussianBlur4847" />
-    </filter>
-    <filter
-       inkscape:collect="always"
-       id="filter4849"
-       x="-0.40412903"
-       width="1.8082581"
-       y="-0.3915"
-       height="1.783">
-      <feGaussianBlur
-         inkscape:collect="always"
-         stdDeviation="2.6487424"
-         id="feGaussianBlur4851" />
-    </filter>
-    <filter
-       inkscape:collect="always"
-       id="filter4853">
-      <feGaussianBlur
-         inkscape:collect="always"
-         stdDeviation="2.6487424"
-         id="feGaussianBlur4855" />
-    </filter>
-    <filter
-       inkscape:collect="always"
-       id="filter4980"
-       x="-0.012474804"
-       width="1.0249496"
-       y="-0.31528276"
-       height="1.6305655">
-      <feGaussianBlur
-         inkscape:collect="always"
-         stdDeviation="3.571244"
-         id="feGaussianBlur4982" />
-    </filter>
-  </defs>
-  <sodipodi:namedview
-     id="base"
-     pagecolor="#ffffff"
-     bordercolor="#666666"
-     borderopacity="1.0"
-     gridtolerance="10000"
-     guidetolerance="10"
-     objecttolerance="10"
-     inkscape:pageopacity="0.0"
-     inkscape:pageshadow="2"
-     inkscape:zoom="1"
-     inkscape:cx="151.32036"
-     inkscape:cy="3.6159398"
-     inkscape:document-units="px"
-     inkscape:current-layer="layer4"
-     showgrid="false"
-     inkscape:window-width="1280"
-     inkscape:window-height="725"
-     inkscape:window-x="0"
-     inkscape:window-y="25" />
-  <metadata
-     id="metadata3402">
-    <rdf:RDF>
-      <cc:Work
-         rdf:about="">
-        <dc:format>image/svg+xml</dc:format>
-        <dc:type
-           rdf:resource="http://purl.org/dc/dcmitype/StillImage" />
-      </cc:Work>
-    </rdf:RDF>
-  </metadata>
-  <g
-     inkscape:label="bg"
-     inkscape:groupmode="layer"
-     id="layer1"
-     transform="translate(-139.07094,-313.30117)"
-     style="display:inline"
-     sodipodi:insensitive="true">
-    <rect
-       style="opacity:1;fill:#ffeeaa;fill-opacity:1;fill-rule:evenodd;stroke:none;stroke-width:4;stroke-linecap:butt;stroke-linejoin:miter;marker:none;marker-start:none;marker-mid:none;marker-end:none;stroke-miterlimit:4;stroke-dasharray:none;stroke-dashoffset:0;stroke-opacity:1;visibility:visible;display:inline;overflow:visible;enable-background:accumulate"
-       id="rect3311"
-       width="531.52441"
-       height="100.33682"
-       x="109.59953"
-       y="290.23679" />
-  </g>
-  <g
-     inkscape:groupmode="layer"
-     id="layer4"
-     inkscape:label="ivle"
-     transform="translate(-139.07094,-313.30117)"
-     sodipodi:insensitive="true">
-    <text
-       xml:space="preserve"
-       style="font-size:12px;font-style:normal;font-weight:normal;fill:#000000;fill-opacity:1;stroke:none;stroke-width:1px;stroke-linecap:butt;stroke-linejoin:miter;stroke-opacity:1;font-family:Bitstream Vera Sans"
-       x="131.42857"
-       y="463.07648"
-       id="text3417"><tspan
-         sodipodi:role="line"
-         id="tspan3419"
-         x="131.42857"
-         y="463.07648"></tspan></text>
-    <g
-       id="g4984">
-      <text
-         sodipodi:linespacing="125%"
-         id="text3407"
-         y="358.55869"
-         x="135.65366"
-         style="font-size:82.79566956px;font-style:normal;font-variant:normal;font-weight:bold;font-stretch:normal;text-align:start;line-height:125%;writing-mode:lr-tb;text-anchor:start;fill:#ffffff;fill-opacity:0.58851677;stroke:none;stroke-width:1px;stroke-linecap:butt;stroke-linejoin:miter;stroke-opacity:0.43137255;font-family:Dyuthi;-inkscape-font-specification:Dyuthi Bold"
-         xml:space="preserve"><tspan
-           y="358.55869"
-           x="135.65366"
-           id="tspan3409"
-           sodipodi:role="line">I V L E</tspan></text>
-      <text
-         sodipodi:linespacing="125%"
-         id="text3421"
-         y="342.62418"
-         x="157.41562"
-         style="font-size:82.79566956px;font-style:normal;font-variant:normal;font-weight:bold;font-stretch:normal;text-indent:0;text-align:start;text-decoration:none;line-height:125%;letter-spacing:normal;word-spacing:normal;text-transform:none;direction:ltr;block-progression:tb;writing-mode:lr-tb;text-anchor:start;opacity:1;fill:#ffffff;fill-opacity:0.58851677;fill-rule:nonzero;stroke:none;stroke-width:1px;stroke-linecap:butt;stroke-linejoin:miter;marker:none;marker-start:none;marker-mid:none;marker-end:none;stroke-miterlimit:4;stroke-dasharray:none;stroke-dashoffset:0;stroke-opacity:0.43137255;visibility:visible;display:inline;overflow:visible;enable-background:accumulate;font-family:Dyuthi;-inkscape-font-specification:Dyuthi Bold"
-         xml:space="preserve"><tspan
-           y="342.62418"
-           x="157.41562"
-           id="tspan3423"
-           sodipodi:role="line">.</tspan></text>
-      <text
-         sodipodi:linespacing="125%"
-         id="text3425"
-         y="342.62418"
-         x="220.80908"
-         style="font-size:82.79566956px;font-style:normal;font-variant:normal;font-weight:bold;font-stretch:normal;text-indent:0;text-align:start;text-decoration:none;line-height:125%;letter-spacing:normal;word-spacing:normal;text-transform:none;direction:ltr;block-progression:tb;writing-mode:lr-tb;text-anchor:start;opacity:1;fill:#ffffff;fill-opacity:0.58851677;fill-rule:nonzero;stroke:none;stroke-width:1px;stroke-linecap:butt;stroke-linejoin:miter;marker:none;marker-start:none;marker-mid:none;marker-end:none;stroke-miterlimit:4;stroke-dasharray:none;stroke-dashoffset:0;stroke-opacity:0.43137255;visibility:visible;display:inline;overflow:visible;enable-background:accumulate;font-family:Dyuthi;-inkscape-font-specification:Dyuthi Bold"
-         xml:space="preserve"><tspan
-           y="342.62418"
-           x="220.80908"
-           id="tspan3427"
-           sodipodi:role="line">.</tspan></text>
-      <text
-         sodipodi:linespacing="125%"
-         id="text3429"
-         y="342.62418"
-         x="273.80267"
-         style="font-size:82.79566956px;font-style:normal;font-variant:normal;font-weight:bold;font-stretch:normal;text-indent:0;text-align:start;text-decoration:none;line-height:125%;letter-spacing:normal;word-spacing:normal;text-transform:none;direction:ltr;block-progression:tb;writing-mode:lr-tb;text-anchor:start;opacity:1;fill:#ffffff;fill-opacity:0.58851677;fill-rule:nonzero;stroke:none;stroke-width:1px;stroke-linecap:butt;stroke-linejoin:miter;marker:none;marker-start:none;marker-mid:none;marker-end:none;stroke-miterlimit:4;stroke-dasharray:none;stroke-dashoffset:0;stroke-opacity:0.43137255;visibility:visible;display:inline;overflow:visible;enable-background:accumulate;font-family:Dyuthi;-inkscape-font-specification:Dyuthi Bold"
-         xml:space="preserve"><tspan
-           y="342.62418"
-           x="273.80267"
-           id="tspan3431"
-           sodipodi:role="line">.</tspan></text>
-    </g>
-  </g>
-  <g
-     inkscape:groupmode="layer"
-     id="layer7"
-     inkscape:label="fullname-shad"
-     transform="translate(-139.07094,-313.30117)">
-    <text
-       xml:space="preserve"
-       style="font-size:31.39154243px;font-style:normal;font-variant:normal;font-weight:normal;font-stretch:normal;text-align:start;line-height:125%;writing-mode:lr-tb;text-anchor:start;opacity:1;fill:#000000;fill-opacity:0.54066988;stroke:none;stroke-width:1px;stroke-linecap:butt;stroke-linejoin:miter;stroke-opacity:1;filter:url(#filter4980);font-family:JackInput;-inkscape-font-specification:JackInput"
-       x="144.54115"
-       y="389.36511"
-       id="text4863"
-       sodipodi:linespacing="125%"
-       transform="matrix(0.6608028,0,0,0.6608028,58.605105,98.23774)"><tspan
-         sodipodi:role="line"
-         id="tspan4865"
-         x="144.54115"
-         y="389.36511">Informatics Virtual Learning Environment</tspan></text>
-  </g>
-  <g
-     inkscape:groupmode="layer"
-     id="layer6"
-     inkscape:label="fullname"
-     transform="translate(-139.07094,-313.30117)"
-     style="display:inline">
-    <text
-       xml:space="preserve"
-       style="font-size:20.74361992px;font-style:normal;font-variant:normal;font-weight:normal;font-stretch:normal;text-align:start;line-height:125%;writing-mode:lr-tb;text-anchor:start;fill:#000000;fill-opacity:1;stroke:none;stroke-width:1px;stroke-linecap:butt;stroke-linejoin:miter;stroke-opacity:1;font-family:JackInput;-inkscape-font-specification:JackInput"
-       x="152.79671"
-       y="354.20972"
-       id="text4858"
-       sodipodi:linespacing="125%"><tspan
-         sodipodi:role="line"
-         id="tspan4860"
-         x="152.79671"
-         y="354.20972">Informatics Virtual Learning Environment</tspan></text>
-  </g>
-</svg>

=== modified file 'image-source/title.svg'
--- image-source/title.svg	2009-02-24 02:02:03 +0000
+++ image-source/title.svg	2009-02-24 04:44:52 +0000
@@ -8,15 +8,15 @@
    xmlns="http://www.w3.org/2000/svg"
    xmlns:sodipodi="http://sodipodi.sourceforge.net/DTD/sodipodi-0.dtd"
    xmlns:inkscape="http://www.inkscape.org/namespaces/inkscape"
-   width="470"
-   height="50"
+   width="330"
+   height="42"
    id="svg3397"
    sodipodi:version="0.32"
    inkscape:version="0.46"
    sodipodi:docname="title.svg"
    inkscape:output_extension="org.inkscape.output.svg.inkscape"
    version="1.0"
-   inkscape:export-filename="/home/matt/uni/ivle/setup-stuff/ivle/webapp/coremedia/images/chrome/title.png"
+   inkscape:export-filename="/home/matt/uni/ivle/new-ui/ivle/webapp/coremedia/images/chrome/title.png"
    inkscape:export-xdpi="90"
    inkscape:export-ydpi="90"
    style="display:inline">
@@ -31,50 +31,6 @@
        id="perspective3405" />
     <filter
        inkscape:collect="always"
-       id="filter4841"
-       x="-0.40412903"
-       width="1.8082581"
-       y="-0.3915"
-       height="1.783">
-      <feGaussianBlur
-         inkscape:collect="always"
-         stdDeviation="2.6487424"
-         id="feGaussianBlur4843" />
-    </filter>
-    <filter
-       inkscape:collect="always"
-       id="filter4845"
-       x="-0.40412903"
-       width="1.8082581"
-       y="-0.3915"
-       height="1.783">
-      <feGaussianBlur
-         inkscape:collect="always"
-         stdDeviation="2.6487424"
-         id="feGaussianBlur4847" />
-    </filter>
-    <filter
-       inkscape:collect="always"
-       id="filter4849"
-       x="-0.40412903"
-       width="1.8082581"
-       y="-0.3915"
-       height="1.783">
-      <feGaussianBlur
-         inkscape:collect="always"
-         stdDeviation="2.6487424"
-         id="feGaussianBlur4851" />
-    </filter>
-    <filter
-       inkscape:collect="always"
-       id="filter4853">
-      <feGaussianBlur
-         inkscape:collect="always"
-         stdDeviation="2.6487424"
-         id="feGaussianBlur4855" />
-    </filter>
-    <filter
-       inkscape:collect="always"
        id="filter4980"
        x="-0.012474804"
        width="1.0249496"
@@ -96,11 +52,11 @@
      objecttolerance="10"
      inkscape:pageopacity="0.0"
      inkscape:pageshadow="2"
-     inkscape:zoom="1"
-     inkscape:cx="186.03804"
-     inkscape:cy="3.058724"
+     inkscape:zoom="1.4142136"
+     inkscape:cx="161.77056"
+     inkscape:cy="8.7618042"
      inkscape:document-units="px"
-     inkscape:current-layer="layer7"
+     inkscape:current-layer="layer6"
      showgrid="false"
      inkscape:window-width="1280"
      inkscape:window-height="725"
@@ -122,22 +78,20 @@
      inkscape:groupmode="layer"
      id="layer1"
      transform="translate(-139.07094,-313.30117)"
-     style="display:none"
-     sodipodi:insensitive="true">
+     style="opacity:0.69777778;display:none">
     <rect
-       style="opacity:1;fill:#ffeeaa;fill-opacity:1;fill-rule:evenodd;stroke:none;stroke-width:4;stroke-linecap:butt;stroke-linejoin:miter;marker:none;marker-start:none;marker-mid:none;marker-end:none;stroke-miterlimit:4;stroke-dasharray:none;stroke-dashoffset:0;stroke-opacity:1;visibility:visible;display:inline;overflow:visible;enable-background:accumulate"
+       style="opacity:1;fill:#aabbff;fill-opacity:1;fill-rule:evenodd;stroke:none;stroke-width:4;stroke-linecap:butt;stroke-linejoin:miter;marker:none;marker-start:none;marker-mid:none;marker-end:none;stroke-miterlimit:4;stroke-dasharray:none;stroke-dashoffset:0;stroke-opacity:1;visibility:visible;display:inline;overflow:visible;enable-background:accumulate"
        id="rect3311"
-       width="531.52441"
-       height="100.33682"
-       x="109.59953"
-       y="290.23679" />
+       width="336.36288"
+       height="48.718002"
+       x="135.76248"
+       y="310.74289" />
   </g>
   <g
      inkscape:groupmode="layer"
      id="layer4"
      inkscape:label="ivle"
-     transform="translate(-139.07094,-313.30117)"
-     sodipodi:insensitive="true">
+     transform="translate(-139.07094,-313.30117)">
     <text
        xml:space="preserve"
        style="font-size:12px;font-style:normal;font-weight:normal;fill:#000000;fill-opacity:1;stroke:none;stroke-width:1px;stroke-linecap:butt;stroke-linejoin:miter;stroke-opacity:1;font-family:Bitstream Vera Sans"
@@ -147,21 +101,55 @@
          sodipodi:role="line"
          id="tspan3419"
          x="131.42857"
-         y="463.07648"></tspan></text>
+         y="463.07648" /></text>
     <g
        id="g4984"
-       transform="translate(-8,-4)">
+       transform="matrix(0.8397741,0,0,0.8397741,17.034512,46.839261)"
+       style="font-style:italic;-inkscape-font-specification:Sans">
       <text
          sodipodi:linespacing="125%"
          id="text3407"
          y="367.55869"
-         x="145.70648"
-         style="font-size:55.29292297px;font-style:italic;font-variant:normal;font-weight:normal;font-stretch:normal;text-align:start;line-height:125%;writing-mode:lr-tb;text-anchor:start;fill:#ffffff;fill-opacity:0.58851677;stroke:none;stroke-width:1px;stroke-linecap:butt;stroke-linejoin:miter;stroke-opacity:0.43137255;font-family:Annifont;-inkscape-font-specification:Annifont Italic"
+         x="183.70648"
+         style="font-size:55.29292297px;font-style:italic;font-variant:normal;font-weight:normal;font-stretch:normal;text-align:start;line-height:125%;writing-mode:lr-tb;text-anchor:start;fill:#ffffff;fill-opacity:0.58851677;stroke:none;stroke-width:1px;stroke-linecap:butt;stroke-linejoin:miter;stroke-opacity:0.43137255;font-family:Annifont;-inkscape-font-specification:Sans"
          xml:space="preserve"><tspan
            y="367.55869"
-           x="145.70648"
+           x="183.70648"
            id="tspan3409"
-           sodipodi:role="line">I V L E</tspan></text>
+           sodipodi:role="line">I</tspan></text>
+      <text
+         sodipodi:linespacing="125%"
+         id="text2402"
+         y="367.55869"
+         x="258.17184"
+         style="font-size:55.29292297px;font-style:italic;font-variant:normal;font-weight:normal;font-stretch:normal;text-align:start;line-height:125%;writing-mode:lr-tb;text-anchor:start;fill:#ffffff;fill-opacity:0.58851677;stroke:none;stroke-width:1px;stroke-linecap:butt;stroke-linejoin:miter;stroke-opacity:0.43137255;display:inline;font-family:Annifont;-inkscape-font-specification:Sans"
+         xml:space="preserve"><tspan
+           y="367.55869"
+           x="258.17184"
+           id="tspan2404"
+           sodipodi:role="line">V</tspan></text>
+      <text
+         sodipodi:linespacing="125%"
+         id="text2406"
+         y="367.55869"
+         x="359.37134"
+         style="font-size:55.29292297px;font-style:italic;font-variant:normal;font-weight:normal;font-stretch:normal;text-align:start;line-height:125%;writing-mode:lr-tb;text-anchor:start;fill:#ffffff;fill-opacity:0.58851677;stroke:none;stroke-width:1px;stroke-linecap:butt;stroke-linejoin:miter;stroke-opacity:0.43137255;display:inline;font-family:Annifont;-inkscape-font-specification:Annifont Italic"
+         xml:space="preserve"><tspan
+           y="367.55869"
+           x="359.37134"
+           id="tspan2408"
+           sodipodi:role="line">L</tspan></text>
+      <text
+         sodipodi:linespacing="125%"
+         id="text2410"
+         y="367.55869"
+         x="443.70648"
+         style="font-size:55.29292297px;font-style:italic;font-variant:normal;font-weight:normal;font-stretch:normal;text-align:start;line-height:125%;writing-mode:lr-tb;text-anchor:start;fill:#ffffff;fill-opacity:0.58851677;stroke:none;stroke-width:1px;stroke-linecap:butt;stroke-linejoin:miter;stroke-opacity:0.43137255;display:inline;font-family:Annifont;-inkscape-font-specification:Sans"
+         xml:space="preserve"><tspan
+           y="367.55869"
+           x="443.70648"
+           id="tspan2412"
+           sodipodi:role="line">E</tspan></text>
     </g>
   </g>
   <g
@@ -176,7 +164,7 @@
        y="389.36511"
        id="text4863"
        sodipodi:linespacing="125%"
-       transform="matrix(0.6608028,0,0,0.6608028,52.605105,88.23774)"><tspan
+       transform="matrix(0.5549251,0,0,0.5549251,67.92911,124.29813)"><tspan
          sodipodi:role="line"
          id="tspan4865"
          x="144.54115"
@@ -190,14 +178,14 @@
      style="display:inline">
     <text
        xml:space="preserve"
-       style="font-size:20.74361992px;font-style:normal;font-variant:normal;font-weight:bold;font-stretch:normal;text-align:start;line-height:125%;writing-mode:lr-tb;text-anchor:start;fill:#000000;fill-opacity:1;stroke:none;stroke-width:1px;stroke-linecap:butt;stroke-linejoin:miter;stroke-opacity:1;font-family:Sawasdee;-inkscape-font-specification:Sawasdee Bold"
-       x="146.79671"
-       y="344.20972"
+       style="font-size:17.4199543px;font-style:normal;font-variant:normal;font-weight:bold;font-stretch:normal;text-align:start;line-height:125%;writing-mode:lr-tb;text-anchor:start;fill:#000000;fill-opacity:1;stroke:none;stroke-width:1px;stroke-linecap:butt;stroke-linejoin:miter;stroke-opacity:1;font-family:Sawasdee;-inkscape-font-specification:Sawasdee Bold"
+       x="147.02878"
+       y="339.96387"
        id="text4858"
        sodipodi:linespacing="125%"><tspan
          sodipodi:role="line"
          id="tspan4860"
-         x="146.79671"
-         y="344.20972">Informatics Virtual Learning Environment</tspan></text>
+         x="147.02878"
+         y="339.96387">Informatics Virtual Learning Environment</tspan></text>
   </g>
 </svg>

=== modified file 'ivle/auth/guest_auth.py'
--- ivle/auth/guest_auth.py	2009-01-15 01:06:33 +0000
+++ ivle/auth/guest_auth.py	2009-02-24 13:29:44 +0000
@@ -32,7 +32,6 @@
 
 from ivle.auth import AuthError
 from ivle.database import User
-from ivle.caps import Role
 
 # XXX: What to put here
 GUEST_UID = 4000
@@ -52,4 +51,4 @@
 
     # Create a guest user
     return User(login=u"guest", fullname=u"Guest Account", nick=u"Guest",
-        role=Role.ANYONE, state=u"no_agreement", unixid=GUEST_UID)
+        state=u"no_agreement", unixid=GUEST_UID)

=== removed file 'ivle/caps.py'
--- ivle/caps.py	2009-01-14 07:17:29 +0000
+++ ivle/caps.py	1970-01-01 00:00:00 +0000
@@ -1,138 +0,0 @@
-# IVLE - Informatics Virtual Learning Environment
-# Copyright (C) 2007-2008 The University of Melbourne
-#
-# This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-#
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
-
-# Module: Capabilities
-# Author: Matt Giuca
-# Date:   15/2/2008
-
-# Provides a Role class which is used for the "rolenm" field of login rows
-# from the database.
-# This also provides a set of Caps (capabilities) and the ability to check if
-# any given role has a given capability.
-
-class Role(object):
-    """
-    Role enumeration class. Provides values for "rolenm" fields, which are
-    enumerable and comparable. Immutable objects.
-
-    This allows you to check if a role has a certain capability by doing a
-    >= comparison on the lowest role with that capability.
-
-    Class Role has a capitalized member for each role, of type Role.
-    eg. Role.STUDENT is a Role object describing the "student" role.
-    """
-    _roles = ["anyone", "student", "marker", "tutor", "lecturer", "admin"]
-    _roles_to_int = {}          # Will be written after class def
-    def __init__(self, role):
-        """Creates a new Role object.
-        role may be any of the following types:
-        * Role: Copies.
-        * str: Role of that name, as given by the str of a Role
-            (These correspond to the entries in the "rolenm" field of the DB).
-            Raises an Exception if the string is not a role.
-        * int: Role of that numeric index, as given by the fromEnum of a Role.
-            (This is the same as calling toEnum).
-        """
-        # Internally, stored as an int
-        if isinstance(role, Role):
-            self._role = role._role
-        elif isinstance(role, basestring):
-            try:
-                self._role = Role._roles_to_int[role.lower()]
-            except KeyError:
-                raise Exception("Not a valid role name: %s" % repr(role))
-        elif isinstance(role, int):
-            if role >= 0 and role < len(Role._roles):
-                self._role = role
-            else:
-                raise Exception("Not a valid role ID: %d" % role)
-    def __str__(self):
-        return Role._roles[self._role]
-    def __repr__(self):
-        return "Role.%s" % Role._roles[self._role].upper()
-    def __cmp__(self, other):
-        return cmp(self._role, other._role)
-    def __hash__(self):
-        return hash(self._role)
-
-    def fromEnum(self):
-        """Returns the int representation of this Role."""
-        return self._role
-    @staticmethod
-    def toEnum(num):
-        """Given an int, returns the Role for that role ID."""
-        return Role(num)
-
-    def hasCap(self, capability):
-        """Given a capability (which is a Role object), returns True if this
-        Role has that capability, False otherwise.
-        """
-        return self >= capability
-
-# Role constants. These are actually members of the Role class, but they are
-# defined here because they actually have the type Role.
-for i in range(0, len(Role._roles)):
-    # XXX This is the only way I could find to write an attribute to a
-    # new-style class. It looks very dodgy.
-    type.__setattr__(Role, Role._roles[i].upper(), Role(i))
-    Role._roles_to_int[Role._roles[i]] = i
-
-### CAPABILITIES LISTING ###
-
-# Set of capabilities, which maps global variables onto Roles.
-# This provides the minimum role level required in order to perform the given
-# capability.
-# (So any role above the role specified can perform this cap).
-
-# Create users (the users are able to log in and accept)
-CAP_CREATEUSER = Role.ADMIN
-# Get details about users
-CAP_GETUSER = Role.LECTURER
-# Change all details of a user
-CAP_UPDATEUSER = Role.ADMIN
-
-# Posting to subject blog for subjects you are teaching
-CAP_BLOGPOST = Role.TUTOR
-
-# Reading submissions (both tutorial and assignment) of student for whom you
-# are a marker
-CAP_READ_MY_STUDENTS_SUBMISSION = Role.TUTOR
-# Reading all students' submissions (in subjects you have this cap for)
-CAP_READ_SUBMISSION = Role.LECTURER
-
-# Reading marks for your own students
-CAP_READ_MY_STUDENTS_MARKS = Role.TUTOR
-# Adding marks info for your own students (can't delete or edit, only add)
-# (You can overwrite marks but old ones will be logged).
-CAP_WRITE_MY_STUDENTS_MARKS = Role.TUTOR
-# Reading marks for all students
-CAP_READ_MARKS = Role.LECTURER
-# Adding marks info for all students
-CAP_WRITE_MARKS = Role.LECTURER
-
-# Reading any student's svn (in subjects you have this cap for)
-CAP_READ_SVN = Role.LECTURER
-# Writing to any student's svn
-CAP_WRITE_SVN = Role.LECTURER
-
-# Create and modify projects and project sets
-CAP_MANAGEPROJECTS = Role.LECTURER
-# Create and modify project groups and assign logins to them
-CAP_MANAGEGROUPS = Role.TUTOR
-
-# "SuperUser" role - certain users are granted "sudo" powers on the Unix
-# system, giving them abilities beyond what is granted here in IVLE.

=== modified file 'ivle/database.py'
--- ivle/database.py	2009-02-23 23:11:26 +0000
+++ ivle/database.py	2009-02-25 02:59:48 +0000
@@ -31,7 +31,6 @@
                          Reference, ReferenceSet, Bool, Storm, Desc
 
 import ivle.conf
-import ivle.caps
 
 __all__ = ['get_store',
             'User',
@@ -39,7 +38,7 @@
             'ProjectSet', 'Project', 'ProjectGroup', 'ProjectGroupMembership',
             'Exercise', 'Worksheet', 'WorksheetExercise',
             'ExerciseSave', 'ExerciseAttempt',
-            'AlreadyEnrolledError', 'TestCase', 'TestSuite', 'TestSuiteVar'
+            'TestCase', 'TestSuite', 'TestSuiteVar'
         ]
 
 def _kwarg_init(self, **kwargs):
@@ -87,7 +86,7 @@
     login = Unicode()
     passhash = Unicode()
     state = Unicode()
-    rolenm = Unicode()
+    admin = Bool()
     unixid = Int()
     nick = Unicode()
     pass_exp = DateTime()
@@ -99,16 +98,6 @@
     studentid = Unicode()
     settings = Unicode()
 
-    def _get_role(self):
-        if self.rolenm is None:
-            return None
-        return ivle.caps.Role(self.rolenm)
-    def _set_role(self, value):
-        if not isinstance(value, ivle.caps.Role):
-            raise TypeError("role must be an ivle.caps.Role")
-        self.rolenm = unicode(value)
-    role = property(_get_role, _set_role)
-
     __init__ = _kwarg_init
 
     def __repr__(self):
@@ -125,12 +114,6 @@
             return None
         return self.hash_password(password) == self.passhash
 
-    def hasCap(self, capability):
-        """Given a capability (which is a Role object), returns True if this
-        User has that capability, False otherwise.
-        """
-        return self.role.hasCap(capability)
-
     @property
     def password_expired(self):
         fieldval = self.pass_exp
@@ -212,7 +195,7 @@
         return store.find(cls, cls.login == unicode(login)).one()
 
     def get_permissions(self, user):
-        if user and user.rolenm == 'admin' or user is self:
+        if user and user.admin or user is self:
             return set(['view', 'edit'])
         else:
             return set()
@@ -239,7 +222,7 @@
         perms = set()
         if user is not None:
             perms.add('view')
-            if user.rolenm == 'admin':
+            if user.admin:
                 perms.add('edit')
         return perms
 
@@ -249,7 +232,7 @@
     id = Int(primary=True, name="semesterid")
     year = Unicode()
     semester = Unicode()
-    active = Bool()
+    state = Unicode()
 
     offerings = ReferenceSet(id, 'Offering.semester_id')
 
@@ -286,23 +269,24 @@
         return "<%s %r in %r>" % (type(self).__name__, self.subject,
                                   self.semester)
 
-    def enrol(self, user):
+    def enrol(self, user, role=u'student'):
         '''Enrol a user in this offering.'''
-        # We'll get a horrible database constraint violation error if we try
-        # to add a second enrolment.
-        if Store.of(self).find(Enrolment,
+        enrolment = Store.of(self).find(Enrolment,
                                Enrolment.user_id == user.id,
-                               Enrolment.offering_id == self.id).count() == 1:
-            raise AlreadyEnrolledError()
-
-        e = Enrolment(user=user, offering=self, active=True)
-        self.enrolments.add(e)
+                               Enrolment.offering_id == self.id).one()
+
+        if enrolment is None:
+            enrolment = Enrolment(user=user, offering=self)
+            self.enrolments.add(enrolment)
+
+        enrolment.active = True
+        enrolment.role = role
 
     def get_permissions(self, user):
         perms = set()
         if user is not None:
             perms.add('view')
-            if user.rolenm in ('admin', 'lecturer'):
+            if user.admin:
                 perms.add('edit')
         return perms
 
@@ -314,6 +298,7 @@
     user = Reference(user_id, User.id)
     offering_id = Int(name="offeringid")
     offering = Reference(offering_id, Offering.id)
+    role = Unicode()
     notes = Unicode()
     active = Bool()
 
@@ -331,9 +316,6 @@
         return "<%s %r in %r>" % (type(self).__name__, self.user,
                                   self.offering)
 
-class AlreadyEnrolledError(Exception):
-    pass
-
 # PROJECTS #
 
 class ProjectSet(Storm):
@@ -435,7 +417,7 @@
     def get_permissions(self, user):
         perms = set()
         if user is not None:
-            if user.rolenm in ('admin', 'lecturer'):
+            if user.admin:
                 perms.add('edit')
                 perms.add('view')
         return perms
@@ -455,12 +437,15 @@
     attempts = ReferenceSet(id, "ExerciseAttempt.worksheetid")
     offering = Reference(offering_id, 'Offering.id')
 
-    # Use worksheet_exercises to get access to the WorksheetExercise objects
-    # binding worksheets to exercises. This is required to access the
+    all_worksheet_exercises = ReferenceSet(id,
+        'WorksheetExercise.worksheet_id')
+
+    # Use worksheet_exercises to get access to the *active* WorksheetExercise
+    # objects binding worksheets to exercises. This is required to access the
     # "optional" field.
-    worksheet_exercises = ReferenceSet(id,
-        'WorksheetExercise.worksheet_id')
-        
+    @property
+    def worksheet_exercises(self):
+        return self.all_worksheet_exercises.find(active=True)
 
     __init__ = _kwarg_init
 

=== modified file 'ivle/dispatch/__init__.py'
--- ivle/dispatch/__init__.py	2009-02-23 05:12:57 +0000
+++ ivle/dispatch/__init__.py	2009-02-24 13:29:44 +0000
@@ -166,10 +166,6 @@
         login = req.user.login
     except AttributeError:
         login = None
-    try:
-        role = req.user.role
-    except AttributeError:
-        role = None
 
     # Log File
     try:
@@ -215,8 +211,7 @@
 
     # Error messages are only displayed is the user is NOT a student,
     # or if there has been a problem logging the error message
-    show_errors = (not publicmode) and ((login and \
-                        str(role) != "student") or logfail)
+    show_errors = (not publicmode) and ((login and req.user.admin) or logfail)
     req.write("""<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN"                 
 	"http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">                                      
 <html xmlns="http://www.w3.org/1999/xhtml">

=== modified file 'ivle/makeuser.py'
--- ivle/makeuser.py	2009-02-02 05:14:04 +0000
+++ ivle/makeuser.py	2009-02-25 07:58:26 +0000
@@ -79,11 +79,7 @@
     """
     users = store.find(ivle.database.User)
     groups = {}
-    for u in users:
-        role = str(u.role)
-        if role not in groups:
-            groups[role] = []
-        groups[role].append(u.login)
+    # TODO: Populate groups with per-offering tutors/lecturers/etc.
     f = open(ivle.conf.svn_conf + ".new", "w")
     f.write("# IVLE SVN Repositories Configuration\n")
     f.write("# Auto-generated on %s\n" % time.asctime())
@@ -197,7 +193,7 @@
         raise Exception("Must run make_jail as root")
     
     # tempdir is for putting backup homes in
-    tempdir = os.path.join(ivle.conf.jail_base, '__temp__')
+    tempdir = os.path.join(ivle.conf.jail_src_base, '__temp__')
     if not os.path.exists(tempdir):
         os.makedirs(tempdir)
     elif not os.path.isdir(tempdir):
@@ -227,10 +223,8 @@
         # Change the ownership of all the files to the right unixid
         logging.debug("chown %s's home directory files to uid %d"
             %(user.login, user.unixid))
-        os.chown(userhomedir, user.unixid, user.unixid)
-        for root, dirs, files in os.walk(userhomedir):
-            for fsobj in dirs + files:
-                os.chown(os.path.join(root, fsobj), user.unixid, user.unixid)
+        os.spawnvp(os.P_WAIT, 'chown', ['chown', '-R', '%d:%d' % (user.unixid,
+                                        user.unixid), userhomedir])
     else:
         # No user jail exists
         # Set up the user's home directory

=== modified file 'ivle/pulldown_subj/__init__.py'
--- ivle/pulldown_subj/__init__.py	2009-01-20 00:58:53 +0000
+++ ivle/pulldown_subj/__init__.py	2009-02-25 02:59:48 +0000
@@ -33,7 +33,7 @@
 from subjecterror import SubjectError
 import ivle.conf
 
-from ivle.database import Subject, Offering, Semester, AlreadyEnrolledError
+from ivle.database import Subject, Offering, Semester
 
 def get_subjects(login):
     """
@@ -79,11 +79,8 @@
         if not offering:
             continue
 
-        try:
-            offering.enrol(user)
-            count += 1
-        except AlreadyEnrolledError:
-            pass
+        offering.enrol(user)
+        count += 1
     return count
 
 # Allow imports to get files from this directory.

=== modified file 'ivle/rpc/decorators.py'
--- ivle/rpc/decorators.py	2009-02-16 06:19:05 +0000
+++ ivle/rpc/decorators.py	2009-02-24 13:29:44 +0000
@@ -18,15 +18,31 @@
            func(req, *args, **kwargs)
         return method_or_die
 
-class require_cap(object):
-    '''Require that the logged in user has the specified capability.'''
-    def __init__(self, cap):
-        self.cap = cap
+def require_admin(func):
+    '''Require that the logged in user is an admin.'''
+    def admin_or_die(req, *args, **kwargs):
+       if not req.user or not req.user.admin:
+            raise Unauthorized()
+       func(req, *args, **kwargs)
+    return admin_or_die
+
+class require_role_anywhere(object):
+    '''Require that the logged in user has a role in any offering.'''
+    def __init__(self, *roles):
+        self.roles = roles
 
     def __call__(self, func):
-        def cap_or_die(req, *args, **kwargs):
-           if not req.user or not req.user.hasCap(self.cap):
+        def role_or_die(req, *args, **kwargs):
+            if not req.user:
                 raise Unauthorized()
-           func(req, *args, **kwargs)
-        return cap_or_die
-
+
+            if req.user.admin:
+                return func(req, *args, **kwargs)
+
+            roles = set((e.role for e in req.user.active_enrolments))
+
+            for role in self.roles:
+                if role in roles:
+                    return func(req, *args, **kwargs)
+            raise Unauthorized()
+        return role_or_die

=== modified file 'ivle/util.py'
--- ivle/util.py	2009-02-11 11:53:38 +0000
+++ ivle/util.py	2009-02-25 03:04:08 +0000
@@ -187,7 +187,7 @@
     try:
         return open(ivle.conf.tos_path).read()
     except IOError:
-        return """<h1>Terms of Service</h1>
+        return """\
 <p><b>*** SAMPLE ONLY ***</b></p>
 <p>This is the text of the IVLE Terms of Service.</p>
 <p>The administrator should create a license file with an appropriate

=== modified file 'ivle/webapp/admin/subject-media/subjects-small.png'
Binary files ivle/webapp/admin/subject-media/subjects-small.png and ivle/webapp/admin/subject-media/subjects-small.png differ
=== modified file 'ivle/webapp/admin/subject-media/subjects.png'
Binary files ivle/webapp/admin/subject-media/subjects.png and ivle/webapp/admin/subject-media/subjects.png differ
=== modified file 'ivle/webapp/admin/subject.py'
--- ivle/webapp/admin/subject.py	2009-02-18 05:25:29 +0000
+++ ivle/webapp/admin/subject.py	2009-02-25 07:47:58 +0000
@@ -36,33 +36,13 @@
 class SubjectsView(XHTMLView):
     '''The view of the list of subjects.'''
     template = 'subjects.html'
-    appname = 'subjects' # XXX
+    tab = 'subjects'
 
     def authorize(self, req):
         return req.user is not None
 
     def populate(self, req, ctx):
-        enrolled_subjects = req.user.subjects
-        unenrolled_subjects = [subject for subject in
-                               req.store.find(Subject)
-                               if subject not in enrolled_subjects]
-
-        ctx['enrolled_subjects'] = []
-        ctx['other_subjects'] = []
-
-        for subject in enrolled_subjects:
-            new_subj = {}
-            new_subj['name'] = subject.name
-            new_subj['url'] = subject.url
-            ctx['enrolled_subjects'].append(new_subj)
-
-        if len(unenrolled_subjects) > 0:
-            for subject in unenrolled_subjects:
-                new_subj = {}
-                new_subj['name'] = subject.name
-                new_subj['url'] = subject.url
-                ctx['other_subjects'].append(new_subj)
-
+        ctx['enrolments'] = req.user.active_enrolments
 
 class Plugin(ViewPlugin, MediaPlugin):
     urls = [
@@ -70,8 +50,9 @@
     ]
 
     tabs = [
-        ('subjects', 'Subjects', 'Announcements and information about the '
-         'subjects you are enrolled in.', 'subjects.png', 'subjects', 5)
+        ('subjects', 'Subjects',
+         'View subject content and complete worksheets',
+         'subjects.png', 'subjects', 5)
     ]
 
     media = 'subject-media'

=== modified file 'ivle/webapp/admin/subjects.html'
--- ivle/webapp/admin/subjects.html	2009-02-23 07:27:55 +0000
+++ ivle/webapp/admin/subjects.html	2009-02-25 07:59:41 +0000
@@ -4,39 +4,17 @@
     <title>Subjects</title>
   </head>
   <body>
+    <h1>Subjects</h1>
     <div id="ivle_padding">
-      <h2>Subjects</h2>
-      <py:if test="(len(enrolled_subjects) > 0)">
-        <h2>Subjects</h2>
-        <ul>
-          <py:for each="subject in enrolled_subjects">
-            <py:choose test="subject['url']">
-              <py:when test="None">
-                <li>${subject['name']} (no home page)</li>
-              </py:when>
-              <py:otherwise>
-                <li><a href="${subject['url']}">${subject['name']}</a></li>
-              </py:otherwise>
-            </py:choose>
-          </py:for>
-        </ul>
-      </py:if>
-      <py:if test="(len(other_subjects) > 0)">
-        <h3>Other Subjects</h3>
-        <p>You are not currently enrolled in these subjects</p>
-        <ul>
-          <py:for each="subject in other_subjects">
-            <py:choose test="subject['url']">
-             <py:when test="None">
-               <li>${subject['name']} (no home page)</li>
-             </py:when>
-             <py:otherwise>
-               <li><a href="${subject['url']}">${subject['name']}</a></li>
-             </py:otherwise>
-            </py:choose>
-          </py:for>
-        </ul>
-      </py:if>
+      <ul>
+        <py:for each="enrolment in enrolments">
+          <li>
+            <a href="/subjects/${enrolment.offering.subject.code}/${enrolment.offering.semester.year}/${enrolment.offering.semester.semester}/+worksheets">
+              ${enrolment.offering.subject.name}
+            </a>
+          </li>
+        </py:for>
+      </ul>
     </div>
   </body>
 </html>

=== modified file 'ivle/webapp/admin/user-settings.html'
--- ivle/webapp/admin/user-settings.html	2009-02-05 02:03:35 +0000
+++ ivle/webapp/admin/user-settings.html	2009-02-25 03:04:08 +0000
@@ -6,9 +6,8 @@
     </script>
   </head>
   <body>
+    <h1>User Profile</h1>
     <div id="ivle_padding">
-
-      <h2>User Profile</h2>
       <p>
         <span id="login"></span>
       </p>

=== modified file 'ivle/webapp/admin/user.py'
--- ivle/webapp/admin/user.py	2009-02-23 05:03:35 +0000
+++ ivle/webapp/admin/user.py	2009-02-25 08:21:38 +0000
@@ -28,7 +28,7 @@
 # (as returned by the get_user action)
 user_fields_list = (
     "login", "state", "unixid", "email", "nick", "fullname",
-    "rolenm", "studentid", "acct_exp", "pass_exp", "last_login",
+    "admin", "studentid", "acct_exp", "pass_exp", "last_login",
     "svn_pass"
 )
 
@@ -56,7 +56,7 @@
 
 class UserSettingsView(XHTMLView):
     template = 'user-settings.html'
-    appname = 'settings'
+    tab = 'settings'
     permission = 'edit'
 
     def __init__(self, req, login):

=== modified file 'ivle/webapp/base/ivle-headings.html'
--- ivle/webapp/base/ivle-headings.html	2009-02-24 02:02:03 +0000
+++ ivle/webapp/base/ivle-headings.html	2009-02-25 09:47:54 +0000
@@ -37,32 +37,36 @@
   </py:match>    
   <py:match path="body[@status!='done']" once="true">
     <body>
-      <div id="ivleheader"></div>
-      <div id="ivleheader_text">
-        <h1>IVLE</h1>
-        <h2>Informatics Virtual Learning Environment</h2>
-        <py:choose>
-          <p py:when="publicmode" class="userhello">Running in public mode.</p>
-          <p py:when="logged_in" class="userhello">
-            <span id="usernick">${nick}</span>
-            (<span class="username">${login}</span>) |
-            <a href="/~${login}/+settings">Settings</a> |
-            <py:choose>
-              <a py:when="defined('help_path')" href="/+help/${help_path}">Help</a>
-              <a py:otherwise="" href="/+help/">Help</a>
-            |</py:choose>
-            <a href="/+logout">Sign out</a>
-          </p>
-          <p py:otherwise="" class="userhello">Not logged in.</p>
-        </py:choose>
-      </div>
-      <div py:if="logged_in" id="ivleheader_tabs">
-        <ul id="apptabs">
-          <py:for each="app in apps_in_tabs">
-            <li py:if="app['this_app']" class="thisapp"><span><span><py:if test="app['has_icon']"><img src="${app['icon_url']}" alt="" /> </py:if><a href="${app['path']}" title="${app['desc']}">${app['name']}</a></span></span></li>
-            <li py:if="not app['this_app']"><span><span><py:if test="app['has_icon']"><img src="${app['icon_url']}" alt="" /> </py:if><a href="${app['path']}" title="${app['desc']}">${app['name']}</a></span></span></li>
-          </py:for>
-        </ul>
+      <div id="ivleheader">
+        <h1>
+          <a href="/">
+            <img src="${title_img}" alt="Informatics Virtual Learning Environment" />
+          </a>
+        </h1>
+        <div id="ivleheader_text">
+          <py:choose>
+            <p py:when="publicmode" class="userhello">Running in public mode.</p>
+            <p py:when="logged_in" class="userhello">
+              <span id="usernick">${nick}</span>
+              (<span class="username">${login}</span>) |
+              <a href="/~${login}/+settings">Settings</a> |
+              <py:choose>
+                <a py:when="defined('help_path')" href="/+help/${help_path}">Help</a>
+                <a py:otherwise="" href="/+help/">Help</a>
+              |</py:choose>
+              <a href="/+logout">Log out</a>
+            </p>
+            <p py:otherwise="" class="userhello">Not logged in.</p>
+          </py:choose>
+        </div>
+        <div py:if="logged_in" id="ivleheader_tabs">
+          <ul id="apptabs">
+            <py:for each="app in apps_in_tabs">
+              <li py:if="app['this_app']" class="thisapp"><a href="${app['path']}" title="${app['desc']}"><span><py:if test="app['has_icon']"><img src="${app['icon_url']}" alt="" /> </py:if>${app['name']}</span></a></li>
+              <li py:if="not app['this_app']"><a href="${app['path']}" title="${app['desc']}"><span><py:if test="app['has_icon']"><img src="${app['icon_url']}" alt="" /> </py:if>${app['name']}</span></a></li>
+            </py:for>
+          </ul>
+        </div>
       </div>
       <div id="ivlebody">
         <div id="ivleview">

=== modified file 'ivle/webapp/base/xhtml.py'
--- ivle/webapp/base/xhtml.py	2009-02-23 13:06:17 +0000
+++ ivle/webapp/base/xhtml.py	2009-02-25 09:15:06 +0000
@@ -85,6 +85,8 @@
 
         ctx['scripts_init'] = req.scripts_init
         ctx['app_template'] = app
+        ctx['title_img'] = media_url(req, CorePlugin,
+                                     "images/chrome/title.png")
         self.populate_headings(req, ctx)
         tmpl = loader.load(os.path.join(os.path.dirname(__file__), 
                                                         'ivle-headings.html'))
@@ -117,8 +119,8 @@
             for tab in plugin.tabs:
                 # tab is a tuple: name, title, desc, icon, path
                 new_app = {}
-                new_app['this_app'] = hasattr(self, 'appname') \
-                                      and tab[0] == self.appname
+                new_app['this_app'] = hasattr(self, 'tab') \
+                                      and tab[0] == self.tab
 
                 # Icon name
                 if tab[3] is not None:

=== modified file 'ivle/webapp/base/xhtmlerror.html'
--- ivle/webapp/base/xhtmlerror.html	2009-02-23 07:27:55 +0000
+++ ivle/webapp/base/xhtmlerror.html	2009-02-25 03:04:08 +0000
@@ -4,8 +4,8 @@
     <title>${exception.codename}</title>
   </head>
   <body>
+    <h1>${exception.codename}</h1>
     <div id="ivle_padding">
-      <h1>${exception.codename}</h1>
       <h3>HTTP error ${exception.code}</h3>
       <p py:if="exception.message">${exception.message}</p>
     </div>

=== modified file 'ivle/webapp/base/xhtmlunauthorized.html'
--- ivle/webapp/base/xhtmlunauthorized.html	2009-02-10 01:09:58 +0000
+++ ivle/webapp/base/xhtmlunauthorized.html	2009-02-25 03:04:08 +0000
@@ -4,8 +4,8 @@
     <title>Unauthorized</title>
   </head>
   <body>
+    <h1>Unauthorized</h1>
     <div id="ivle_padding">
-      <h1>Unauthorized</h1>
       <p>You are not allowed to view this part of IVLE.</p>
     </div>
   </body>

=== modified file 'ivle/webapp/console/__init__.py'
--- ivle/webapp/console/__init__.py	2009-02-16 01:19:44 +0000
+++ ivle/webapp/console/__init__.py	2009-02-25 07:47:58 +0000
@@ -5,7 +5,7 @@
 
 
 class ConsoleView(XHTMLView):
-    appname = 'console'
+    tab = 'console'
     help = 'Console'
 
     plugin_scripts = {'ivle.webapp.console': ['console.js']}
@@ -28,8 +28,8 @@
         ('console', ConsoleView),
     ]
     tabs = [
-        ('console', 'Console', 'A Python console where you can try out code '
-         'without having to save and run it.', 'console.png', 'console', 3)
+        ('console', 'Console', 'Try out your code in a Python console',
+         'console.png', 'console', 3)
     ]
     overlays = [
         ConsoleOverlay,

=== modified file 'ivle/webapp/console/media/console.css'
--- ivle/webapp/console/media/console.css	2008-03-15 12:39:10 +0000
+++ ivle/webapp/console/media/console.css	2009-02-25 08:35:26 +0000
@@ -1,7 +1,6 @@
 #console_body {
     background-color: white;
     position: absolute;
-    font-size: 150%;
     color: black;
     padding: 0;
     width: 100%;
@@ -23,7 +22,7 @@
 #console_body.windowpane #console_body2 {
     /* Need some padding in windowpane mode */
     /* (In full mode, things are positioned absolutely) */
-    padding: 0.5em;
+    padding: 0.2em;
 }
 /* Heading bar */
 #console_heading {
@@ -52,12 +51,11 @@
     /*margin-bottom: 0.5em;*/
     position: absolute;
     /* 2em is to accomodate the header bar, and some padding */
-    top: 2em;
+    top: 3.4em;
     /* 2.5em is to accomodate the input bar, and padding either side */
     bottom: 2.5em;
-    /* left and right 0.5em are just for horizontal padding */
-    left: 0.5em;
-    right: 0.5em;
+    left: 0;
+    right: 0;
     /* Display scroll bars */
     overflow: scroll;
     /* Some browsers support overflow-x and overflow-y. In this case, use
@@ -66,14 +64,28 @@
      * horizontal scroll bar).
      */
     overflow-x: auto;
+    padding: 0.5em;
+    /* On some displays, monospace font is much smaller than everything else.
+     * We just set the console font to 13px to appear a bit bigger (this is
+     * the same size as the editor's font).
+     */
+    font-size: 13px;
 }
 #console_body.windowpane #console_output {
     /* Don't want this absolutely positioned in windowpane mode */
     position: static;
+    /* Make it a bit higher up, because the header is thinner in this mode */
+    top: 2em;
     /* Give it an explicit height instead */
     height: 15em;
     margin-bottom: 0.5em;
 }
+
+#console_body.windowpane.maximal, #console_body.windowpane #console_heading {
+    -moz-border-radius-topleft: 10px;
+    -moz-border-radius-topright: 10px;
+}
+
 #console_input {
     position: absolute;
     /* Input bar goes along the bottom. Pad all sides */
@@ -81,6 +93,7 @@
     right: 0.5em;
     bottom: 0.5em;
 }
+
 #console_body.windowpane #console_input {
     /* Don't want this absolutely positioned in windowpane mode */
     position: static;
@@ -89,14 +102,10 @@
     font-family: monospace;
 }
 #console_inputText {
-    background-color: white;
-    font-size: 80%;
-    color: black;
+    font-family: monospace;
     width: 75%;
 }
-#console_inputText.disabled {
-    background-color: #aaa;
-}
+
 /* console filler takes up the same space as console_body.windowpane
  * but its position is relative, not fixed.
  * This avoids having the console overlap with the bottom 28em of page
@@ -117,14 +126,12 @@
     right: 0;
 }
 #console_filler.maximal {
-    /* total height of console = 20em + 1em + 0.5em + 0.5em + 0.5em + 1em + borders
-     */
-    height: 24em;
+    /* Trial and error is useful to work this out. */
+    height: 14em;
 }
 #console_filler.minimal {
-    /* total height of console = 1em + 0.5em + 0.5em
-     */
-    height: 2em;
+    /* Trial and error is useful to work this out. */
+    height: 1em;
 }
 *.errorMsg {
     color: red;

=== modified file 'ivle/webapp/console/media/console.js'
--- ivle/webapp/console/media/console.js	2009-02-05 00:47:10 +0000
+++ ivle/webapp/console/media/console.js	2009-02-23 14:02:21 +0000
@@ -256,14 +256,13 @@
     {
         var inputline = inputbox;
         inputbox = null;
-        var graytimer = null;
     }
     else
     {
-        GLOBAL_inputbox = inputbox;     /* For timer */
+        /* Disable the text box */
+        inputbox.setAttribute("disabled", "disabled");
+
         var inputline = inputbox.value + "\n";
-        var graytimer = setTimeout("GLOBAL_inputbox.setAttribute(\"class\", "
-            + "\"disabled\");", 100);
     }
     var output = document.getElementById("console_output");
     {
@@ -283,15 +282,12 @@
     var args = {"ivle.op": "chat", "kind": which, "key": server_key, "text":inputline};
     var callback = function(xhr)
         {
-            console_response(inputbox, graytimer, inputline, xhr.responseText);
+            console_response(inputbox, inputline, xhr.responseText);
         }
-    /* Disable the text box */
-    if (inputbox != null)
-        inputbox.setAttribute("disabled", "disabled");
     ajax_call(callback, "console", "service", args, "POST");
 }
 
-function console_response(inputbox, graytimer, inputline, responseText)
+function console_response(inputbox, inputline, responseText)
 {
     try
     {
@@ -350,8 +346,7 @@
         }
         var callback = function(xhr)
             {
-                console_response(inputbox, graytimer,
-                                 null, xhr.responseText);
+                console_response(inputbox, null, xhr.responseText);
             }
         if (interrupted)
         {
@@ -383,9 +378,7 @@
     if (inputbox != null)
     {
         /* Re-enable the text box */
-        clearTimeout(graytimer);
         inputbox.removeAttribute("disabled");
-        inputbox.removeAttribute("class");
         interrupted = false;
     }
 

=== modified file 'ivle/webapp/console/overlay.html'
--- ivle/webapp/console/overlay.html	2009-02-09 22:47:11 +0000
+++ ivle/webapp/console/overlay.html	2009-02-25 08:23:59 +0000
@@ -4,8 +4,9 @@
       
   <body py:strip="">
     <div id="console_body" py:attrs="start_body_attrs">
-      <div id="console_heading">Python Console<span py:if="windowpane" 
-          class="console_button minimize">
+      <h1 py:if="not windowpane">Python Console</h1>
+      <div id="console_heading" py:if="windowpane">Python Console
+        <span class="console_button minimize">
           <a onclick="console_minimize()"
               title="Minimize the Python console">
             <img src="${minimize_path}" alt="Minimize" />

=== added file 'ivle/webapp/coremedia/images/chrome/h1-bg.png'
Binary files ivle/webapp/coremedia/images/chrome/h1-bg.png and ivle/webapp/coremedia/images/chrome/h1-bg.png differ
=== modified file 'ivle/webapp/coremedia/images/chrome/title.png'
Binary files ivle/webapp/coremedia/images/chrome/title.png and ivle/webapp/coremedia/images/chrome/title.png differ
=== modified file 'ivle/webapp/coremedia/ivle.css'
--- ivle/webapp/coremedia/ivle.css	2009-02-24 02:02:03 +0000
+++ ivle/webapp/coremedia/ivle.css	2009-02-25 09:47:54 +0000
@@ -29,38 +29,54 @@
     color: #a00;
 }
 
+a img {
+    border: none;
+}
+
+/* h1s are styled specially to a small golden 3D bar */
+#ivlebody h1 {
+    background-color: #fea;
+    background-image: url(../ivle.webapp.core/images/chrome/h1-bg.png);
+    background-repeat: repeat-y;
+    margin: 0;
+    padding: 0.5em;
+    border-top: white 2px solid;        /* Top edge of 3D effect */
+    border-bottom: #863 1px solid;      /* Bottom edge of 3D effect */
+    font-size: 1em;
+    font-weight: bold;
+}
+
+#ivlebody h2 {
+    font-size: 1.4em;
+}
+
 /** Special styles for header section **/
 
 #ivleheader {
     position: absolute;
-    background-color: #fea;
-    background-image: url(../ivle.webapp.core/images/chrome/topbar-hashes.png);
+    background-color: #abf;
+    background-image: url(../ivle.webapp.core/images/chrome/topbar-slashes.png);
     background-repeat: repeat-x;
     padding: 0;
     border-bottom: 1px solid black;
-    /* IMPORTANT: This 5.3em is hard-coded elsewhere in this file. Search for
+    /* IMPORTANT: This 4em is hard-coded elsewhere in this file. Search for
      * #ivleheader.height to update it. */
-    height: 5.3em;
+    height: 4em;
     left: 0;
     right: 0;
     z-index: 1;     /* Background to all, but the body */
 }
 
+#ivleheader h1 {
+    margin: 0;
+    padding: 8px;
+}
+
 #ivleheader_text {
-    /* Pad left and right. */
     position: absolute;
-    padding: 0 1em;
-    background-image: url(../ivle.webapp.core/images/chrome/title.png);
-    background-repeat: no-repeat;
-    background-position: 10px 10px;
-    height: 60px;
-    left: 0;
-    right: 0;
-    z-index: 3;     /* In front */
-}
-
-#ivleheader_text h1, #ivleheader_text h2 {
-    display: none;
+    padding: 0.5em;
+    top: 0;
+    right: 0.5em;
 }
 
 /* Note: The body has NO PADDING by default (unlike plain HTML).
@@ -72,7 +88,7 @@
     /* This is positioned absolutely so that its content can be positioned
      * absolutely as well, and so height: 100% really means it. */
     position: absolute;
-    top: 5.3em;             /* == #ivleheader.height */
+    top: 4em;               /* == #ivleheader.height */
     bottom: 0;
     width: 100%;
     z-index: 0;     /* Behind header */
@@ -80,16 +96,15 @@
 
 /* Apps can optionally have a <div id="ivle_padding"> as the outermost
  * element, if 1em of padding is desired (otherwise there will be no padding).
+ * Note: No top padding, since the first element should have a margins (p or
+ * h2).
  */
 #ivle_padding {
-    padding: 1em;
+    padding: 0 1em 1em 1em;
 }
 
-.userhello {
+p.userhello {
     font-size: .8em;
-    top: 0;
-    text-align: right;
-    padding: 11px 0;
     margin: 0;
 }
 .userhello .username {
@@ -101,14 +116,8 @@
  */
 #ivleheader_tabs {
     position: absolute;
-    height: 5.3em;          /* == #ivleheader.height */
-    top: 0;
-    left: 0;
-    right: 0;
-    width: 100%;
-    padding: 0;
-    margin: 0;
-    z-index: 2;     /* In front of background, behind text */
+    right: 1em;
+    bottom: 0;
 }
 
 #ivleheader_tabs p {
@@ -120,18 +129,12 @@
     margin: 0;
 }
 
-/* The apptabs ul is aligned to the bottom of #ivleheader_tabs */
 ul#apptabs {
-    position: absolute;
-    text-align: right;
-    bottom: 0;
-    right: 1em;             /* == #ivleheader.padding-right */
-    padding: 0;
     margin: 0;
 }
+
 ul#apptabs li {
     display: inline;
-    padding: 3px;
     /*border: 1px solid black;*/
     /*background-color: #fc6;*/
     background-image: url(../ivle.webapp.core/images/chrome/tab-left.png);
@@ -140,14 +143,14 @@
     padding: 0;
     padding-left: 4px;
 }
-ul#apptabs li span {
+ul#apptabs li a {
     background-image: url(../ivle.webapp.core/images/chrome/tab-right.png);
     background-repeat: no-repeat;
     background-position: top right;
     padding: 0;
     padding-right: 4px;
 }
-ul#apptabs li span span {
+ul#apptabs li a span {
     background-image: url(../ivle.webapp.core/images/chrome/tab-mid.png);
     background-repeat: repeat-x;
     padding: 0 3px;
@@ -156,10 +159,10 @@
     /*background-color: #fd8;*/
     background-image: url(../ivle.webapp.core/images/chrome/tab-hover-left.png);
 }
-ul#apptabs li:hover span {
+ul#apptabs li:hover a {
     background-image: url(../ivle.webapp.core/images/chrome/tab-hover-right.png);
 }
-ul#apptabs li:hover span span {
+ul#apptabs li:hover a span {
     background-image: url(../ivle.webapp.core/images/chrome/tab-hover-mid.png);
 }
 /* Currently active app tab */
@@ -168,10 +171,10 @@
     /*border-bottom: 1px solid white;*/
     background-image: url(../ivle.webapp.core/images/chrome/tab-selected-left.png);
 }
-ul#apptabs li.thisapp span {
+ul#apptabs li.thisapp a {
     background-image: url(../ivle.webapp.core/images/chrome/tab-selected-right.png);
 }
-ul#apptabs li.thisapp span span {
+ul#apptabs li.thisapp a span {
     background-image: url(../ivle.webapp.core/images/chrome/tab-selected-mid.png);
 }
 

=== modified file 'ivle/webapp/filesystem/browser/__init__.py'
--- ivle/webapp/filesystem/browser/__init__.py	2009-02-18 05:30:16 +0000
+++ ivle/webapp/filesystem/browser/__init__.py	2009-02-25 07:47:58 +0000
@@ -40,7 +40,7 @@
     The view for the browser
     """
     template = 'template.html'
-    appname = 'files'
+    tab = 'files'
     help = 'Filesystem/Browser'
 
     def authorize(self, req):
@@ -67,12 +67,19 @@
         if localpath is None:
             raise NotFound()
 
+        if isinstance(localpath, unicode):
+            localpath = localpath.encode('utf-8')
+
         # Start writing data
 
         # FIXME: This isn't completely reliable! We're not inside the jail, so we
         # can't know the type for sure. This is now only used for adding a / to the
         # end of displayed paths, so I'm leaving this although it will often break.
-        isdir = os.path.isdir(localpath)
+        try:
+            isdir = os.path.isdir(localpath)
+        except OSError:
+            isdir = False
+
         ctx['isdir'] = isdir
         self.gen_path(req, ctx)
         self.gen_actions(req, ctx)
@@ -80,8 +87,9 @@
         # The page title should contain the name of the file being browsed
         ctx['title'] = self.path.rsplit('/', 1)[-1]
 
-        ctx['fileservice_action'] = util.make_path(os.path.join("fileservice", req.path))
-        ctx['filename'] = cgi.escape(req.path)
+        ctx['fileservice_action'] = util.make_path(os.path.join("fileservice",
+                                                                self.path))
+        ctx['filename'] = cgi.escape(self.path)
 
     #TODO: Move all this logic into the template
     def gen_path(self, req, ctx):
@@ -170,8 +178,8 @@
     ]
 
     tabs = [
-        ('files', 'Files', 'Gives you access to all of your files and lets '
-         'you download, upload, edit and run them.', 'browser.png', 'files', 1)
+        ('files', 'Files', 'Access and edit your files',
+         'browser.png', 'files', 1)
     ]
 
     cookies = {'clipboard': None}

=== modified file 'ivle/webapp/filesystem/browser/media/browser.css'
--- ivle/webapp/filesystem/browser/media/browser.css	2008-07-22 00:21:49 +0000
+++ ivle/webapp/filesystem/browser/media/browser.css	2009-02-25 09:35:36 +0000
@@ -29,22 +29,25 @@
     left: 0;
     right: 0;
     top: 0;
-    /* height == h2.margin + h2.line-height
-     *              + max(h2.margin, p.margin) + p.line-height + p.margin
-     *        == 1.1*0.7em + 1.1*1.2em + 1.1*0.7em + 1.1em + 0.7em
-     *        == 4.6em */
-    height: 4.6em;
-    padding: 0.5em;
-    background-color: #aaf;
+    /* Approximate height of the components */
+    height: 5em;
+    padding: 0;
+    background-color: #ddd;
 }
 
-#topbar div {
-    margin: 0.2em;
+/* We don't want the bold-ness of a regular golden H1 */
+#topbar #path {
+    font-weight: normal;
 }
 
 #topbar #path a {
     font-weight: bold;
 }
+
+#actions {
+    padding: 0.3em 0.5em;
+    border-top: white 1px solid;        /* Top edge of 3D effect */
+}
 /* class "choice" is for all choices, enabled and disabled. This is for both
  * the <a> actions and <option> actions.
  * class "disabled" is for disabled choices.
@@ -68,10 +71,6 @@
     color: #aaa;
 }
 
-#actions2 p {
-    margin: 0;
-}
-
 /* One of the actions2s is shown at runtime, depending on the context.
  * We will show moreactions if we need it. */
 #actions1 #moreactions_area, #actions2 #actions2_file,
@@ -85,9 +84,9 @@
     position: absolute;
     left: 0;
     right: 0;
-    /* top == #topbar.height + #topbar.padding * 2 */
-    top: 5.6em;
+    /* top == #topbar.height */
+    top: 5em;
     /* Space for the mimimised console */
-    bottom: 3.5em;
+    bottom: 2em;
 }
 

=== modified file 'ivle/webapp/filesystem/browser/template.html'
--- ivle/webapp/filesystem/browser/template.html	2009-02-22 08:11:40 +0000
+++ ivle/webapp/filesystem/browser/template.html	2009-02-25 09:35:36 +0000
@@ -6,68 +6,58 @@
 </head>
 <body>
   <div id="topbar">
-    <div id="path">
-      <span>
-        <py:for each="segment in paths">/<a href="${segment['href_path']}" title="Navigate to ${segment['nav_path']}">${segment['path']}</a></py:for><py:if test="isdir">/</py:if><py:if test="revno is not None"> (revision ${revno})</py:if>
-      </span>
-    </div>
-    <div id="actions1">
-      <a id="act_open" class="disabled">Open</a> :
-      <a id="act_serve"
-          title="View this file on the web, running your code if this is a CGI file"
-          class="disabled" target="_blank">Serve</a> :
-      <a id="act_run" title="Run the selected Python file in the console"
-          class="disabled">Run</a> :
-      <a id="act_download" class="choice">Download</a> :
-      <a title="Refresh the current page" onclick="refresh()"
-          class="choice">Refresh</a>
-      <span id="moreactions_area"> :
-        <select id="moreactions" onchange="handle_moreactions()"
-            onblur="handle_moreactions()">
-          <option class="moreactions" value="top"
-              selected="selected">More actions...</option>
-          <py:for each="(name, disablement, group) in moreactions">
-
-            <optgroup label="${name}">
-
-              <py:choose test="disablement">
-
-                <py:when test="True">
-                  <py:for each="(id, bits) in group">
-                    <option id="act_${id}" class="disabled" disabled="disabled"
-                        title="${bits[1]}" value="${id}">${bits[0]}</option>
-                  </py:for>
-                </py:when>
-
-                <py:otherwise>
-                  <py:for each="(id, bits) in group">
-                    <option id="act_${id}" title="${bits[1]}" 
-                        value="${id}">${bits[0]}</option>
-                  </py:for>                
-                </py:otherwise>
-
-              </py:choose>
-
-            </optgroup>
-          </py:for>
-        </select>
-      </span>
-    </div>
-    <div id="actions2">
-      <form id="actions2_directory"
-          target="upload_iframe"
-          action="${fileservice_action}"
-          enctype="multipart/form-data" method="post">
-        <p>
-          <span id="selectallpanel">
-          Select:
-            <a onclick="action_selectall(true)"
-                title="Select all files in this directory">All</a> :
-            <a onclick="action_selectall(false)"
-                title="Deselect all files in this directory">None</a>
-          </span>
-
-          <span style="display:none" id="uploadpanel">|
+    <h1 id="path">
+      <py:for each="segment in paths">/<a href="${segment['href_path']}" title="Navigate to ${segment['nav_path']}">${segment['path']}</a></py:for><py:if test="isdir">/</py:if><py:if test="revno is not None"> (revision ${revno})</py:if>
+    </h1>
+    <div id="actions">
+      <span id="actions1">
+        <a id="act_open" class="disabled">Open</a> :
+        <a id="act_serve"
+            title="View this file on the web, running your code if this is a CGI file"
+            class="disabled" target="_blank">Serve</a> :
+        <a id="act_run" title="Run the selected Python file in the console"
+            class="disabled">Run</a> :
+        <a id="act_download" class="choice">Download</a> :
+        <a title="Refresh the current page" onclick="refresh()"
+            class="choice">Refresh</a>
+        <span id="moreactions_area"> :
+          <select id="moreactions" onchange="handle_moreactions()"
+              onblur="handle_moreactions()">
+            <option class="moreactions" value="top"
+                selected="selected">More actions...</option>
+            <py:for each="(name, disablement, group) in moreactions">
+
+              <optgroup label="${name}">
+
+                <py:choose test="disablement">
+
+                  <py:when test="True">
+                    <py:for each="(id, bits) in group">
+                      <option id="act_${id}" class="disabled" disabled="disabled"
+                          title="${bits[1]}" value="${id}">${bits[0]}</option>
+                    </py:for>
+                  </py:when>
+
+                  <py:otherwise>
+                    <py:for each="(id, bits) in group">
+                      <option id="act_${id}" title="${bits[1]}" 
+                          value="${id}">${bits[0]}</option>
+                    </py:for>                
+                  </py:otherwise>
+
+                </py:choose>
+
+              </optgroup>
+            </py:for>
+          </select>
+        </span>
+      </span> <!-- actions1 -->
+      <span id="actions2">
+        <form id="actions2_directory"
+            target="upload_iframe"
+            action="${fileservice_action}"
+            enctype="multipart/form-data" method="post">
+          <span style="display:none" id="uploadpanel">
             <label for="upload_file">Upload file:</label>
             <input type="hidden" value="putfile" name="action" />
             <input type="hidden" value="" name="path" />
@@ -81,20 +71,20 @@
                refreshing the page. (It will refresh the listing). -->
           <iframe onload="upload_callback()" style="display: none;"
               name="upload_iframe" id="upload_iframe"></iframe>
-        </p>
-      </form>
-      <p id="actions2_file">
-        <input type="button" id="save_button" value="Save" onclick="save_file('${filename}')" />
-        <input type="button" id="saveas_button" value="Save As..." onclick="save_file_as('${filename}')" />
-        <select id="highlighting_select" onchange="highlighting_changed(this)">
-          <option value="text">Text</option>
-          <option value="python">Python</option>
-          <option value="html">HTML</option>
-          <option value="javascript">JavaScript</option>
-          <option value="css">CSS</option>
-        </select>
-      </p>
-    </div>
+        </form>
+        <span id="actions2_file">
+          <input type="button" id="save_button" value="Save" onclick="save_file('${filename}')" />
+          <input type="button" id="saveas_button" value="Save As..." onclick="save_file_as('${filename}')" />
+          <select id="highlighting_select" onchange="highlighting_changed(this)">
+            <option value="text">Text</option>
+            <option value="python">Python</option>
+            <option value="html">HTML</option>
+            <option value="javascript">JavaScript</option>
+            <option value="css">CSS</option>
+          </select>
+        </span>
+      </span> <!-- actions2 -->
+    </div> <!-- actions -->
   </div>
   <!-- Body. The JavaScript places content here relevant to the path -->
   <div id="filesbody">

=== modified file 'ivle/webapp/filesystem/diff/template.html'
--- ivle/webapp/filesystem/diff/template.html	2009-02-05 05:24:19 +0000
+++ ivle/webapp/filesystem/diff/template.html	2009-02-25 03:04:08 +0000
@@ -4,8 +4,8 @@
     <title>Diff</title>
   </head>
   <body>
+    <h1>Diff</h1>
     <div id="ivle_padding">
-      <h1>Diff</h1>
       <py:for each="file in files">
         <p><b>File:</b> ${file}</p>
         <pre py:replace="files[file]" />

=== modified file 'ivle/webapp/forum/__init__.py'
--- ivle/webapp/forum/__init__.py	2009-02-18 02:02:06 +0000
+++ ivle/webapp/forum/__init__.py	2009-02-25 08:21:38 +0000
@@ -30,7 +30,7 @@
 import ivle.config
 
 class ForumView(XHTMLView):
-    appname = 'forum'
+    tab = 'forum'
 
     def __init__(self, req, path):
         self.path = path
@@ -61,7 +61,7 @@
     if user.email != None:
         email = quote(user.email)
 
-    role = quote(str(user.role))
+    role = 'admin' if user.admin else 'student' # XXX
 
     hashtext = login + nick + email + role + secret
     hash = hashlib.md5(hashtext).hexdigest()

=== modified file 'ivle/webapp/groups/__init__.py'
--- ivle/webapp/groups/__init__.py	2009-02-12 05:52:25 +0000
+++ ivle/webapp/groups/__init__.py	2009-02-25 08:21:38 +0000
@@ -23,7 +23,6 @@
 
 # TODO Does not distinguish between current and past subjects.
 
-from ivle import caps
 from ivle.database import Subject
 
 from ivle.webapp.base.plugins import ViewPlugin, MediaPlugin
@@ -34,7 +33,7 @@
     The groups view
     """
     template = 'template.html'
-    appname = 'groups' # XXX
+    tab = 'groups'
 
     def authorize(self, req):
         return req.user is not None
@@ -46,8 +45,10 @@
         # Show a group panel per enrolment
         ctx['get_user_groups'] = req.user.get_groups
         ctx['enrolments'] = req.user.active_enrolments
+
+        roles = set((e.role for e in req.user.active_enrolments))
         ctx['manage_subjects'] = req.store.find(Subject) if \
-              req.user.hasCap(caps.CAP_MANAGEGROUPS) else []
+              req.user.admin or 'tutor' in roles or 'lecturer' in roles else []
 
 
 class Plugin(ViewPlugin, MediaPlugin):

=== modified file 'ivle/webapp/groups/template.html'
--- ivle/webapp/groups/template.html	2009-02-05 21:31:34 +0000
+++ ivle/webapp/groups/template.html	2009-02-25 03:04:08 +0000
@@ -4,13 +4,14 @@
     <title>Groups</title>
   </head>
   <body>
+    <h1>Project Groups</h1>
     <div id="ivle_padding">
-      <p py:if="not enrolments">Error: You are not currently enrolled in any subjects.</p>
+      <p py:if="enrolments.count() == 0">Error: You are not currently enrolled in any subjects.</p>
       <py:for each="enrolment in enrolments">
         <div id="subject${enrolment.offering.id}" class="subject">
-          <h1>${enrolment.offering.subject.name}</h1>
+          <h2>${enrolment.offering.subject.name}</h2>
           <py:for each="group in get_user_groups(enrolment.offering)">
-            <h2>${group.nick} (${group.name})</h2>
+            <h3>${group.nick} (${group.name})</h3>
             <!-- Need proper test (for invite/membership). -->
             <!-- <py:choose test="group">
               <py:when test="True"> -->
@@ -28,7 +29,7 @@
                 </p>
               </py:otherwise>
             </py:choose>-->
-            <h3>Members</h3>
+            <h4>Members</h4>
             <ul>
               <py:for each="member in group.members">
                 <li>${member.fullname} (${member.login})</li>
@@ -39,7 +40,7 @@
       </py:for>
       <py:if test="manage_subjects">
         <hr />
-        <h1>Group Administration</h1>
+        <h2>Group Administration</h2>
         <label for="subject_select">Subject:</label>
         <select id="subject_select">
           <py:for each="subject in manage_subjects">

=== modified file 'ivle/webapp/help/__init__.py'
--- ivle/webapp/help/__init__.py	2009-02-18 07:00:19 +0000
+++ ivle/webapp/help/__init__.py	2009-02-25 07:47:58 +0000
@@ -29,7 +29,7 @@
 
 class HelpView(XHTMLView):
     """Shows the help file for the specified path."""
-
+    tab = 'help'
     template = 'helpview.html'
 
     def __init__(self, req, path):
@@ -59,7 +59,7 @@
 
 class HelpToCView(XHTMLView):
     """Displays the help Table of Contents."""
-    appname = 'help'
+    tab = 'help'
     template = 'toc.html'
 
     def authorize(self, req):
@@ -78,7 +78,7 @@
     ]
 
     tabs = [
-        ('help', 'Help', 'IVLE help pages', 'help.png', '+help', 100)
+        ('help', 'Help', 'Get help with IVLE', 'help.png', '+help', 100)
     ]
 
     media = 'media'

=== modified file 'ivle/webapp/help/helpview.html'
--- ivle/webapp/help/helpview.html	2009-02-11 10:27:56 +0000
+++ ivle/webapp/help/helpview.html	2009-02-25 03:04:08 +0000
@@ -5,8 +5,8 @@
     <title>Help</title>
   </head>
   <body>
+    <h1>Help</h1>
     <div id="ivle_padding">
-      <h1>Help</h1>
       <xi:include href="${helpfile}"/>
     </div>
   </body>

=== modified file 'ivle/webapp/help/toc.html'
--- ivle/webapp/help/toc.html	2009-02-11 11:56:16 +0000
+++ ivle/webapp/help/toc.html	2009-02-25 03:04:08 +0000
@@ -4,8 +4,8 @@
     <title>Help</title>
   </head>
   <body>
+    <h1>Help</h1>
     <div id="ivle_padding">
-      <h1>Help</h1>
       <py:def function="show_toc(toc_part, path)">
         <py:for each="part in toc_part">
           <py:if test="isinstance(toc_part[part], dict)">

=== modified file 'ivle/webapp/security/logout.html'
--- ivle/webapp/security/logout.html	2009-02-23 07:27:55 +0000
+++ ivle/webapp/security/logout.html	2009-02-25 03:04:08 +0000
@@ -4,8 +4,9 @@
     <title>Log out</title>
   </head>
   <body>
+    <h1>Confirm log out</h1>
     <div id="ivle_padding">
-      <h3>Are you sure you want to log out?</h3>
+      <p>Are you sure you want to log out?</p>
       <p>
         <form action="${path}" method="POST">
           <input type="submit" value="Log out" />

=== modified file 'ivle/webapp/tos/accept.html'
--- ivle/webapp/tos/accept.html	2009-02-16 04:42:34 +0000
+++ ivle/webapp/tos/accept.html	2009-02-25 03:04:08 +0000
@@ -4,6 +4,7 @@
     <title>Terms of Service</title>
   </head>
   <body>
+    <h1>Terms of Service</h1>
     <div id="ivle_padding">
       <p>Welcome, <strong>${user.fullname}</strong>.</p>
       <p>

=== modified file 'ivle/webapp/tos/media/tos.js'
--- ivle/webapp/tos/media/tos.js	2009-02-16 05:00:36 +0000
+++ ivle/webapp/tos/media/tos.js	2009-02-25 01:28:18 +0000
@@ -102,7 +102,7 @@
     else
     {
         /* Refresh the page; as the user is now (apparently) logged in */
-        window.location.href = parse_url(window.location.href).args["url"];
+        window.location.href = parse_url(window.location.href).args["url"] || "/";
     }
 }
 

=== modified file 'ivle/webapp/tos/view.html'
--- ivle/webapp/tos/view.html	2009-02-16 04:42:34 +0000
+++ ivle/webapp/tos/view.html	2009-02-25 03:04:08 +0000
@@ -4,6 +4,7 @@
     <title>Terms of Service</title>
   </head>
   <body>
+    <h1>Terms of Service</h1>
     <div id="ivle_padding">
       <p>When you first logged into IVLE, you agreed to the following Terms of
 Service:</p>

=== modified file 'ivle/webapp/tutorial/__init__.py'
--- ivle/webapp/tutorial/__init__.py	2009-02-23 07:25:19 +0000
+++ ivle/webapp/tutorial/__init__.py	2009-02-25 08:21:38 +0000
@@ -69,7 +69,7 @@
 class OfferingView(XHTMLView):
     '''The view of the index of worksheets for an offering.'''
     template = 'templates/subjectmenu.html'
-    appname = 'tutorial' # XXX
+    tab = 'subjects' # XXX
     permission = 'view'
 
     def __init__(self, req, subject, year, semester):
@@ -146,7 +146,7 @@
 class WorksheetView(XHTMLView):
     '''The view of a worksheet with exercises.'''
     template = 'templates/worksheet.html'
-    appname = 'tutorial' # XXX
+    tab = 'subjects'
     permission = 'view'
 
     def __init__(self, req, subject, year, semester, worksheet):
@@ -361,7 +361,8 @@
     #Save the exercise details to the Table of Contents
 
     loader = genshi.template.TemplateLoader(".", auto_reload=True)
-    tmpl = loader.load(os.path.join(os.path.dirname(__file__), "exercise.html"))
+    tmpl = loader.load(os.path.join(os.path.dirname(__file__),
+        "templates/exercise.html"))
     ex_stream = tmpl.generate(curctx)
     return {'name': exercise.name,
             'complete': curctx['complete_class'],
@@ -383,7 +384,7 @@
     into XML directly from RST."""
     permission = "edit"
     template = "templates/worksheet_edit.html"
-    appname = "Edit Worksheet"
+    tab = "subjects"
 
     def __init__(self, req, **kwargs):
     

=== modified file 'ivle/webapp/tutorial/media/tutorial.css'
--- ivle/webapp/tutorial/media/tutorial.css	2009-02-06 06:20:29 +0000
+++ ivle/webapp/tutorial/media/tutorial.css	2009-02-25 10:05:26 +0000
@@ -7,6 +7,7 @@
     border-right: 2px solid gray;
     border-top: 1px solid gray;
     border-bottom: 2px solid gray;
+    -moz-border-radius: 10px;
 }
 
 .exercisebox, .runoutput {
@@ -63,6 +64,21 @@
     margin-left: 1.5em;
 }
 
+/* Cancel out the main stylesheet's h1 (the golden bar at the top).
+ * Make it just look like the default (HTML4) style sheet's h1, but a bit
+ * smaller.
+ */
+#ivle_padding h1 {
+    background: none;
+    margin: 0;
+    padding: 0;
+    margin: 0.67em 0;
+    border: 0;
+    font-weight: bolder;
+    /* font-size: 2em; (standard) */
+    font-size: 1.6em;
+}
+
 h1 a, h2 a {
     color: black;
     text-decoration: none;
@@ -215,18 +231,17 @@
 /* 'Note' callouts */
 div.note
 {
-  border-right:   #87ceeb 1px solid;
-  padding-right: 4px;
-  border-top: #87ceeb 1px solid;
-  padding-left: 4px;
-  padding-bottom: 4px;
+  padding: 4px;
   margin: 2px 5% 10px;
-  border-left: #87ceeb 1px solid;
-  padding-top: 4px;
-  border-bottom: #87ceeb 1px solid;
   font-style: normal;
-  font-family: verdana, arial;
+  font-family: sans-serif;
   background-color: #b0c4de;
+  -moz-border-radius: 10px;
+}
+
+div.note p.admonition-title
+{
+  font-weight: bold;
 }
 
 table.rst-example { border: 1px solid black; }

=== modified file 'ivle/webapp/tutorial/service.py'
--- ivle/webapp/tutorial/service.py	2009-02-23 04:37:56 +0000
+++ ivle/webapp/tutorial/service.py	2009-02-24 23:50:36 +0000
@@ -209,52 +209,6 @@
         return {"result": "ok"}
 
 
-def generate_exerciselist(worksheet, req, worksheetdata):
-    """Runs through the worksheetstream, generating the appropriate
-    WorksheetExercises, and de-activating the old ones."""
-    exercises = []
-    # Turns the worksheet into an xml stream, and then finds all the 
-    # exercise nodes in the stream.
-    worksheetdata = genshi.XML(worksheetdata)
-    for kind, data, pos in worksheetdata:
-        if kind is genshi.core.START:
-            # Data is a tuple of tag name and a list of name->value tuples
-            if data[0] == 'exercise':
-                src = ""
-                optional = False
-                for attr in data[1]:
-                    if attr[0] == 'src':
-                        src = attr[1]
-                    if attr[0] == 'optional':
-                        optional = attr[1] == 'true'
-                if src != "":
-                    exercises.append((src, optional))
-    ex_num = 0
-    # Set all current worksheet_exercises to be inactive
-    db_worksheet_exercises = req.store.find(WorksheetExercise,
-        WorksheetExercise.worksheet_id == worksheet.id)
-    for worksheet_exercise in db_worksheet_exercises:
-        worksheet_exercise.active = False
-    
-    for exerciseid, optional in exercises:
-        worksheet_exercise = req.store.find(WorksheetExercise,
-            WorksheetExercise.worksheet_id == worksheet.id,
-            Exercise.id == WorksheetExercise.exercise_id,
-            Exercise.id == exerciseid).one()
-        if worksheet_exercise is None:
-            exercise = req.store.find(Exercise,
-                Exercise.id == exerciseid
-            ).one()
-            if exercise is None:
-                raise NotFound()
-            worksheet_exercise = WorksheetExercise()
-            worksheet_exercise.worksheet_id = worksheet.id
-            worksheet_exercise.exercise_id = exercise.id
-            req.store.add(worksheet_exercise)
-        worksheet_exercise.active = True
-        worksheet_exercise.seq_no = ex_num
-        worksheet_exercise.optional = optional
-
 
 # Note that this is the view of an existing worksheet. Creation is handled
 # by OfferingRESTView (as offerings have worksheets)
@@ -266,7 +220,7 @@
         # XXX: Lecturers should be allowed to add worksheets Only to subjects
         #      under their control
         if user is not None:
-            if user.rolenm == 'admin':
+            if user.admin:
                 return set(['save'])
             else:
                 return set()
@@ -295,12 +249,11 @@
     @named_operation('save')
     def save(self, req, name, assessable, data, format):
         """Takes worksheet data and saves it."""
-        generate_exerciselist(self.context, req, data)
-        
         self.context.name = unicode(name)
         self.context.assessable = self.convert_bool(assessable)
         self.context.data = unicode(data)
         self.context.format = unicode(format)
+        ivle.worksheet.update_exerciselist(self.context)
         
         return {"result": "ok"}
 
@@ -312,7 +265,7 @@
         # XXX: Lecturers should be allowed to add worksheets Only to subjects
         #      under their control
         if user is not None:
-            if user.rolenm == 'admin':
+            if user.admin:
                 return set(['edit'])
             else:
                 return set()
@@ -353,7 +306,7 @@
         # This call is added for clarity, as the worksheet is implicitly added.        
         req.store.add(new_worksheet)
         
-        generate_exerciselist(new_worksheet, req, data)
+        ivle.worksheet.update_exerciselist(new_worksheet)
         
         return {"result": "ok"}
 

=== modified file 'ivle/webapp/tutorial/templates/subjectmenu.html'
--- ivle/webapp/tutorial/templates/subjectmenu.html	2009-02-23 07:25:19 +0000
+++ ivle/webapp/tutorial/templates/subjectmenu.html	2009-02-25 03:04:08 +0000
@@ -4,8 +4,8 @@
     <title>Worksheets - ${subject.name}</title>
   </head>
   <body>
+    <h1>Worksheets for ${subject.name}</h1>
     <div id="ivle_padding">
-      <h1>Worksheets for ${subject.name}</h1>
       <ul id="tutorial-toc">
         <py:for each="worksheet in worksheets">
           <li>

=== modified file 'ivle/webapp/tutorial/templates/worksheet.html'
--- ivle/webapp/tutorial/templates/worksheet.html	2009-02-23 07:25:19 +0000
+++ ivle/webapp/tutorial/templates/worksheet.html	2009-02-25 03:04:08 +0000
@@ -11,8 +11,8 @@
     </script>
   </head>
   <body>
+    <h1>${worksheet.name} in ${subject.name}</h1>
     <div id="ivle_padding">
-      <h1>${worksheet.name} in ${subject.name}</h1>
       <!-- Display the Table Of Contents -->
       <div id="tutorial-toc">
         <h2>Contents</h2>

=== modified file 'ivle/webapp/tutorial/templates/worksheet_add.html'
--- ivle/webapp/tutorial/templates/worksheet_add.html	2009-02-23 07:25:19 +0000
+++ ivle/webapp/tutorial/templates/worksheet_add.html	2009-02-25 03:04:08 +0000
@@ -10,8 +10,8 @@
     </script>
   </head>
   <body>
+    <h1>New worksheet for ${subject.name}</h1>
     <div id="ivle_padding">
-      <h1>New worksheet for ${subject.name}</h1>
       <div id='worksheet_input'>
         <label for='worksheet_identifier'>Unique Identifier:</label>
         <input type='text' id='worksheet_identifier' class='worksheet_id' />

=== modified file 'ivle/webapp/tutorial/templates/worksheet_edit.html'
--- ivle/webapp/tutorial/templates/worksheet_edit.html	2009-02-23 07:25:19 +0000
+++ ivle/webapp/tutorial/templates/worksheet_edit.html	2009-02-25 09:15:06 +0000
@@ -11,14 +11,15 @@
     </script>
   </head>
   <body>
+    <h1>Edit ${worksheet.name} in ${subject.name}</h1>
     <div id="ivle_padding">
-      <h1>Edit ${worksheet.name} in ${subject.name}</h1>
       <div id='worksheet_input'>
         <label for='worksheet_name'>Name:</label>
         <input type='text' id='worksheet_name' class='worksheet_area' value='${worksheet.name}' />
         <br />
         <label for='worksheet_asses'>Assessable:</label>
-        <input type='checkbox' name='asses' id='worksheet_asses' checked='${worksheet.assessable}' />
+        <input type='checkbox' name='asses' id='worksheet_asses'
+               py:attrs="{'checked': 'checked' if worksheet.assessable else None}" />
         <br />
         <label for='worksheet_format'>Worksheet Format:</label>
         <select name='format' id='worksheet_format'>

=== modified file 'ivle/webapp/tutorial/templates/worksheets_edit.html'
--- ivle/webapp/tutorial/templates/worksheets_edit.html	2009-02-23 07:25:19 +0000
+++ ivle/webapp/tutorial/templates/worksheets_edit.html	2009-02-25 03:04:08 +0000
@@ -10,8 +10,8 @@
     </script>
   </head>
   <body>
+    <h1>Editing worksheets for ${subject.name}</h1>
     <div id="ivle_padding">
-      <h1>Editing worksheets for ${subject.name}</h1>
       <table id='worksheets_table'>
         <tr><th>Name</th><th>Actions</th></tr>
       <py:for each="worksheet in worksheets">

=== modified file 'ivle/webapp/userservice/__init__.py'
--- ivle/webapp/userservice/__init__.py	2009-02-18 05:55:43 +0000
+++ ivle/webapp/userservice/__init__.py	2009-02-25 01:18:09 +0000
@@ -138,12 +138,13 @@
 import cjson
 
 import ivle.database
-from ivle import (util, chat, caps)
+from ivle import (util, chat)
 from ivle.conf import (usrmgt_host, usrmgt_port, usrmgt_magic)
 from ivle.webapp.security import get_user_details
 import ivle.pulldown_subj
 
-from ivle.rpc.decorators import require_method, require_cap
+from ivle.rpc.decorators import require_method, require_role_anywhere, \
+                                require_admin
 
 from ivle.auth import AuthError, authenticate
 import urllib
@@ -160,7 +161,7 @@
 # (as returned by the get_user action)
 user_fields_list = (
     "login", "state", "unixid", "email", "nick", "fullname",
-    "rolenm", "studentid", "acct_exp", "pass_exp", "last_login",
+    "admin", "studentid", "acct_exp", "pass_exp", "last_login",
     "svn_pass"
 )
 
@@ -270,14 +271,14 @@
     req.write(cjson.encode(response))
 
 create_user_fields_required = [
-    'login', 'fullname', 'rolenm'
+    'login', 'fullname',
 ]
 create_user_fields_optional = [
-    'password', 'nick', 'email', 'studentid'
+    'admin', 'password', 'nick', 'email', 'studentid'
 ]
 
 @require_method('POST')
-@require_cap(caps.CAP_UPDATEUSER)
+@require_admin
 def handle_create_user(req, fields):
     """Create a new user, whose state is no_agreement.
     This does not create the user's jail, just an entry in the database which
@@ -296,9 +297,8 @@
         fullname    - The name of the user for results and/or other official
                       purposes.
                       STRING REQUIRED
-        rolenm      - The user's role. Must be one of "anyone", "student",
-                      "tutor", "lecturer", "admin".
-                      STRING/ENUM REQUIRED
+        admin       - Whether the user is an admin.
+                      BOOLEAN REQUIRED
         studentid   - If supplied and not None, the student id of the user for
                       results and/or other official purposes.
                       STRING OPTIONAL
@@ -330,7 +330,7 @@
     'password', 'nick', 'email'
 ]
 update_user_fields_admin = [
-    'password', 'nick', 'email', 'rolenm', 'unixid', 'fullname',
+    'password', 'nick', 'email', 'admin', 'unixid', 'fullname',
     'studentid'
 ]
 
@@ -338,10 +338,10 @@
 def handle_update_user(req, fields):
     """Update a user's account details.
     This can be done in a limited form by any user, on their own account,
-    or with full powers by a user with CAP_UPDATEUSER on any account.
+    or with full powers by an admin user on any account.
     """
-    # Only give full powers if this user has CAP_UPDATEUSER
-    fullpowers = req.user.hasCap(caps.CAP_UPDATEUSER)
+    # Only give full powers if this user is an admin.
+    fullpowers = req.user.admin
     # List of fields that may be changed
     fieldlist = (update_user_fields_admin if fullpowers
         else update_user_fields_anyone)
@@ -392,8 +392,8 @@
     module is willing to give up, EXCEPT the following fields:
         svn_pass
     """
-    # Only give full powers if this user has CAP_GETUSER
-    fullpowers = req.user.hasCap(caps.CAP_GETUSER)
+    # Only give full powers if this user is an admin
+    fullpowers = req.user.admin
 
     try:
         login = fields.getfirst('login')
@@ -426,8 +426,6 @@
     """
     # For the moment we're only able to query ourselves
     fullpowers = False
-    ## Only give full powers if this user has CAP_GETUSER
-    ##fullpowers = req.user.hasCap(caps.CAP_GETUSER)
 
     try:
         user = ivle.database.User.get_by_login(req.store,
@@ -478,9 +476,9 @@
                               'subj_name': offering.subject.name,
                               'year': offering.semester.year,
                               'semester': offering.semester.semester,
-                              'active': offering.semester.active
+                              'active': True # XXX: Eliminate from protocol.
                              } for offering in subject.offerings
-                                    if offering.semester.active])
+                                    if offering.semester.state == 'current'])
     req.content_type = "text/plain"
     req.write(response)
 
@@ -515,7 +513,7 @@
     req.write(response)
 
 @require_method('POST')
-@require_cap(caps.CAP_MANAGEGROUPS)
+@require_role_anywhere('tutor', 'lecturer')
 def handle_create_group(req, fields):
     """Required cap: CAP_MANAGEGROUPS
     Creates a project group in a specific project set
@@ -625,7 +623,7 @@
     req.write(response)
 
 @require_method('POST')
-@require_cap(caps.CAP_MANAGEGROUPS)
+@require_role_anywhere('tutor', 'lecturer')
 def handle_assign_group(req, fields):
     """ Required cap: CAP_MANAGEGROUPS
     Assigns a user to a project group

=== modified file 'ivle/worksheet.py'
--- ivle/worksheet.py	2009-02-23 04:37:56 +0000
+++ ivle/worksheet.py	2009-02-24 23:50:36 +0000
@@ -23,9 +23,12 @@
 This module provides functions for tutorial and worksheet computations.
 """
 
-from storm.locals import And, Asc, Desc
+from storm.locals import And, Asc, Desc, Store
+import genshi
+
 import ivle.database
-from ivle.database import ExerciseAttempt, ExerciseSave, Worksheet
+from ivle.database import ExerciseAttempt, ExerciseSave, Worksheet, \
+                          WorksheetExercise, Exercise
 
 __all__ = ['get_exercise_status', 'get_exercise_stored_text',
            'get_exercise_attempts', 'get_exercise_attempt',
@@ -190,3 +193,50 @@
             if done: mand_done += 1
 
     return mand_done, mand_total, opt_done, opt_total
+
+def update_exerciselist(worksheet):
+    """Runs through the worksheetstream, generating the appropriate
+    WorksheetExercises, and de-activating the old ones."""
+    exercises = []
+    # Turns the worksheet into an xml stream, and then finds all the 
+    # exercise nodes in the stream.
+    worksheetdata = genshi.XML(worksheet.data)
+    for kind, data, pos in worksheetdata:
+        if kind is genshi.core.START:
+            # Data is a tuple of tag name and a list of name->value tuples
+            if data[0] == 'exercise':
+                src = ""
+                optional = False
+                for attr in data[1]:
+                    if attr[0] == 'src':
+                        src = attr[1]
+                    if attr[0] == 'optional':
+                        optional = attr[1] == 'true'
+                if src != "":
+                    exercises.append((src, optional))
+    ex_num = 0
+    # Set all current worksheet_exercises to be inactive
+    db_worksheet_exercises = Store.of(worksheet).find(WorksheetExercise,
+        WorksheetExercise.worksheet_id == worksheet.id)
+    for worksheet_exercise in db_worksheet_exercises:
+        worksheet_exercise.active = False
+    
+    for exerciseid, optional in exercises:
+        worksheet_exercise = Store.of(worksheet).find(WorksheetExercise,
+            WorksheetExercise.worksheet_id == worksheet.id,
+            Exercise.id == WorksheetExercise.exercise_id,
+            Exercise.id == exerciseid).one()
+        if worksheet_exercise is None:
+            exercise = Store.of(worksheet).find(Exercise,
+                Exercise.id == exerciseid
+            ).one()
+            if exercise is None:
+                raise NotFound()
+            worksheet_exercise = WorksheetExercise()
+            worksheet_exercise.worksheet_id = worksheet.id
+            worksheet_exercise.exercise_id = exercise.id
+            Store.of(worksheet).add(worksheet_exercise)
+        worksheet_exercise.active = True
+        worksheet_exercise.seq_no = ex_num
+        worksheet_exercise.optional = optional
+

=== modified file 'setup/util.py'
--- setup/util.py	2009-02-18 11:13:17 +0000
+++ setup/util.py	2009-02-25 02:01:45 +0000
@@ -321,4 +321,6 @@
         "bin/ivle-config",
         "bin/ivle-createdatadirs",
         "bin/ivle-buildjail",
+        "bin/ivle-addexercise",
+        "bin/ivle-cloneworksheets",
     ]

=== added file 'userdb/migrations/20090224-01.sql'
--- userdb/migrations/20090224-01.sql	1970-01-01 00:00:00 +0000
+++ userdb/migrations/20090224-01.sql	2009-02-24 13:29:44 +0000
@@ -0,0 +1,7 @@
+BEGIN;
+ALTER TABLE enrolment ADD COLUMN role TEXT NOT NULL CHECK (role IN
+                         ('student', 'tutor', 'lecturer')) DEFAULT 'student';
+ALTER TABLE login ADD COLUMN admin BOOLEAN NOT NULL DEFAULT false;
+UPDATE login SET admin = (rolenm = 'admin');
+ALTER TABLE login DROP COLUMN rolenm;
+COMMIT;

=== added file 'userdb/migrations/20090225-01.sql'
--- userdb/migrations/20090225-01.sql	1970-01-01 00:00:00 +0000
+++ userdb/migrations/20090225-01.sql	2009-02-25 01:18:09 +0000
@@ -0,0 +1,11 @@
+BEGIN;
+DROP TRIGGER deactivate_semester_enrolments ON semester;
+DROP FUNCTION deactivate_semester_enrolments_update();
+DROP TRIGGER confirm_active_semester ON enrolment;
+DROP FUNCTION confirm_active_semester_insertupdate();
+
+ALTER TABLE semester ADD COLUMN state TEXT NOT NULL DEFAULT 'current';
+ALTER TABLE semester ADD CONSTRAINT semester_state_check CHECK (state IN ('disabled', 'past', 'current', 'future'));
+UPDATE semester SET state='past' WHERE active='false';
+ALTER TABLE semester DROP COLUMN active;
+COMMIT;

=== modified file 'userdb/users.sql'
--- userdb/users.sql	2009-02-23 01:40:04 +0000
+++ userdb/users.sql	2009-02-25 04:44:54 +0000
@@ -8,9 +8,7 @@
     state	VARCHAR NOT NULL CHECK (state in ('no_agreement', 'pending',
                                               'enabled', 'disabled'))
                                  DEFAULT 'no_agreement',
-    rolenm      VARCHAR NOT NULL CHECK (rolenm in ('anyone', 'student',
-                                                   'marker', 'tutor',
-                                                   'lecturer', 'admin')),
+    admin       BOOLEAN NOT NULL DEFAULT false,
     unixid      INT UNIQUE DEFAULT nextval('login_unixid_seq') NOT NULL,
     nick        VARCHAR NOT NULL,
     pass_exp    TIMESTAMP,
@@ -38,25 +36,11 @@
     semesterid  SERIAL PRIMARY KEY NOT NULL,
     year        CHAR(4) NOT NULL,
     semester    CHAR(1) NOT NULL,
-    active      BOOL NOT NULL,
+    state       TEXT NOT NULL CHECK (state IN ('disabled', 'past',
+                                    'current', 'future')) DEFAULT 'current',
     UNIQUE (year, semester)
 );
 
-CREATE OR REPLACE FUNCTION deactivate_semester_enrolments_update()
-RETURNS trigger AS '
-    BEGIN
-        IF OLD.active = true AND NEW.active = false THEN
-            UPDATE enrolment SET active=false WHERE offeringid IN (
-            SELECT offeringid FROM offering WHERE offering.semesterid = NEW.semesterid);
-        END IF;
-        RETURN NULL;
-    END;
-' LANGUAGE 'plpgsql';
-
-CREATE TRIGGER deactivate_semester_enrolments
-    AFTER UPDATE ON semester
-    FOR EACH ROW EXECUTE PROCEDURE deactivate_semester_enrolments_update();
-
 CREATE TABLE offering (
     offeringid  SERIAL PRIMARY KEY NOT NULL,
     subject     INT4 REFERENCES subject (subjectid) NOT NULL,
@@ -129,6 +113,8 @@
 CREATE TABLE enrolment (
     loginid     INT4 REFERENCES login (loginid),
     offeringid  INT4 REFERENCES offering (offeringid),
+    role        TEXT NOT NULL CHECK (role IN ('student', 'tutor',
+                                              'lecturer')) DEFAULT 'student',
     result      INT,
     special_result VARCHAR,
     supp_result INT,
@@ -138,23 +124,6 @@
     PRIMARY KEY (loginid,offeringid)
 );
 
-CREATE OR REPLACE FUNCTION confirm_active_semester_insertupdate()
-RETURNS trigger AS '
-    DECLARE
-        active BOOL;
-    BEGIN
-        SELECT semester.active INTO active FROM offering, semester WHERE offeringid=NEW.offeringid AND semester.semesterid = offering.semesterid;
-        IF NOT active AND NEW.active = true THEN
-            RAISE EXCEPTION ''cannot have active enrolment for % in offering %, as the semester is inactive'', NEW.loginid, NEW.offeringid;
-        END IF;
-        RETURN NEW;
-    END;
-' LANGUAGE 'plpgsql';
-
-CREATE TRIGGER confirm_active_semester
-    BEFORE INSERT OR UPDATE ON enrolment
-    FOR EACH ROW EXECUTE PROCEDURE confirm_active_semester_insertupdate();
-
 CREATE TABLE assessed (
     assessedid  SERIAL PRIMARY KEY NOT NULL,
     loginid     INT4 REFERENCES login (loginid),

